/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFComplementary.h"

using namespace CodeWorker;

BNFComplementary::BNFComplementary(DtaBNFScript* pBNFScript, GrfBlock* pParent, bool bContinue) : BNFStepper(pBNFScript, pParent, bContinue) {}

BNFComplementary::~BNFComplementary() {}

SEQUENCE_INTERRUPTION_LIST BNFComplementary::executeInternal(DtaScriptVariable& visibility) {
	int iLocation = CGRuntime::getLocation();
	int iImplicitCopyPosition = _pBNFScript->skipEmptyChars(visibility);
	int iNotEmptyLocation = CGRuntime::getLocation();
	_iStepLocation = 0;
	SEQUENCE_INTERRUPTION_LIST result = GrfBlock::executeInternal(visibility);
	if (result == NO_INTERRUPTION) {
		if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
		CGRuntime::setLocation(iLocation);
		if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
		result = BREAK_INTERRUPTION;
	} else if (result != CONTINUE_INTERRUPTION) {
//		CGRuntime::setLocation(iNotEmptyLocation);
		std::string sText;
		if (_iStepLocation > iNotEmptyLocation) {
			CGRuntime::setLocation(_iStepLocation);
			sText = CGRuntime::getLastReadChars(_iStepLocation - iNotEmptyLocation);
		} else {
			int iChar = CGRuntime::readCharAsInt();
			if (iChar < 0) {
				if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
				CGRuntime::setLocation(iLocation);
				if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
				return BREAK_INTERRUPTION;
			}
			sText = std::string(1, (char) iChar);
		}
		result = NO_INTERRUPTION;
		if (!_listOfConstants.empty()) {
			bool bMatch = false;
			for (std::vector<std::string>::const_iterator i = _listOfConstants.begin(); i != _listOfConstants.end(); i++) {
				bMatch = ((*i) == sText);
				if (bMatch) break;
			}
			if (!bMatch) {
				if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
				CGRuntime::setLocation(iLocation);
				if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
				return BREAK_INTERRUPTION;
			}
		}
		if (_pVariableToAssign != 0L) {
			DtaScriptVariable* pVariable = visibility.getVariable(*_pVariableToAssign);
			if (_iClauseReturnType == BNFClause::LIST_RETURN_TYPE) {
				pVariable->pushItem(sText);
			} else {
				pVariable->setValue(sText.c_str());
			}
		}
		if (iImplicitCopyPosition >= 0) _pBNFScript->writeBinaryData(sText.c_str(), sText.size());
	}
	return result;
}

void BNFComplementary::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	std::vector<GrfCommand*>::const_iterator i = getCommands().begin();
	if (i != getCommands().end()) {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
		theCompilerEnvironment.getBody().endl();
		int iCursor = theCompilerEnvironment.newCursor();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseLocation_" << iCursor << " = CGRuntime::getLocation();";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseImplicitCopy_" << iCursor << " = theEnvironment.skipEmptyChars();";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseNotEmptyLocation_" << iCursor << " = CGRuntime::getLocation();";
		theCompilerEnvironment.getBody().endl();
		int iOldBNFStepperCursor = theCompilerEnvironment.getBNFStepperCursor();
		theCompilerEnvironment.setBNFStepperCursor(iCursor);
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseNextLocation_" << iCursor << " = 0;";
		theCompilerEnvironment.getBody().endl();
		for (;;) {
			(*i)->compileCpp(theCompilerEnvironment);
			i++;
			if (i == getCommands().end()) break;
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
		}
		int j = getNbCommands() - 1;
		while (j > 0) {
			theCompilerEnvironment.decrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
			j--;
		}
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerClauseSuccess = !_compilerClauseSuccess;";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tCGRuntime::setLocation(_compilerClauseNotEmptyLocation_" << iCursor << ");";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tint iChar = CGRuntime::readCharAsInt();";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t_compilerClauseSuccess = (iChar >= 0);";
		theCompilerEnvironment.getBody().endl();
		if (!_listOfConstants.empty()) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tif (_compilerClauseSuccess) switch(iChar) {";
			theCompilerEnvironment.getBody().endl();
			for (std::vector<std::string>::const_iterator i = _listOfConstants.begin(); i != _listOfConstants.end(); i++) {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t\tcase " << (int) ((unsigned char) ((*i)[0])) << ": // '" << CGRuntime::composeCLikeString(*i) << "'";
				theCompilerEnvironment.getBody().endl();
			}
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t\t\tbreak;";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t\tdefault: _compilerClauseSuccess = false;";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t}";
			theCompilerEnvironment.getBody().endl();
		}
		theCompilerEnvironment.incrementIndentation();
		if (_pVariableToAssign != 0L) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tchar tcText[] = {'\\0', '\\0'};";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\ttcText[0] = (char) iChar;";
			theCompilerEnvironment.getBody().endl();
			if (_iClauseReturnType == BNFClause::LIST_RETURN_TYPE) {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t";
				_pVariableToAssign->compileCppForBNFSet(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".pushItem(tcText);";
				theCompilerEnvironment.getBody().endl();
			} else {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\t";
				_pVariableToAssign->compileCppForBNFSet(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".setValue(tcText);";
				theCompilerEnvironment.getBody().endl();
			}
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
		}
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (!_compilerClauseSuccess) {";
		theCompilerEnvironment.getBody().endl();
		if (_bContinue) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tRuntime::throwBNFExecutionError(";
			theCompilerEnvironment.getBody().writeString(toString());
			theCompilerEnvironment.getBody() << ");";
		} else {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tCGRuntime::setLocation(_compilerClauseLocation_" << iCursor << ");";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tif (_compilerClauseImplicitCopy_" << iCursor << " >= 0) CGRuntime::resizeOutputStream(_compilerClauseImplicitCopy_" << iCursor << ");";
		}
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "} else if (_compilerClauseImplicitCopy_" << iCursor << " >= 0) {";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tunsigned char cChar = (unsigned char) _compilerClauseChar_" << iCursor << ";";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tCGRuntime::writeBinaryData((const char*) &cChar, 1);";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.decrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
		theCompilerEnvironment.setBNFStepperCursor(iOldBNFStepperCursor);
	}
}

std::string BNFComplementary::toString() const {
	std::string sText = "~" + getCommands()[0]->toString() + DtaBNFScript::constantsToString(_listOfConstants) + DtaBNFScript::assignmentToString(_pVariableToAssign);
	if (_bContinue) sText = "#continue " + sText;
	return sText;
}
