/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFConjunction.h"

using namespace CodeWorker;

BNFConjunction::BNFConjunction(DtaBNFScript* pBNFScript, GrfBlock* pParent) : _pBNFScript(pBNFScript), GrfBlock(pParent) {}

BNFConjunction::~BNFConjunction() {
}

bool BNFConjunction::isABNFCommand() const { return true; }

SEQUENCE_INTERRUPTION_LIST BNFConjunction::execute(DtaScriptVariable& visibility) {
	SEQUENCE_INTERRUPTION_LIST result;
	int iCursor = CGRuntime::getLocation();
	int iImplicitCopyPosition = ((_pBNFScript->implicitCopy()) ? CGRuntime::getOutputLocation() : -1);
	result = GrfBlock::executeInternal(visibility);
	if ((result != NO_INTERRUPTION) && (result != CONTINUE_INTERRUPTION)) {
		CGRuntime::setLocation(iCursor);
		if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
	}
	return result;
}

void BNFConjunction::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	std::vector<GrfCommand*>::const_iterator i = getCommands().begin();
	if (i != getCommands().end()) {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
		theCompilerEnvironment.getBody().endl();
		int iCursor = theCompilerEnvironment.newCursor();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseCursor_" << iCursor << " = CGRuntime::getLocation();";
		theCompilerEnvironment.getBody().endl();
		for (;;) {
			(*i)->compileCpp(theCompilerEnvironment);
			i++;
			if (i == getCommands().end()) break;
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
		}
		int j = getNbCommands() - 1;
		while (j > 0) {
			theCompilerEnvironment.decrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
			j--;
		}
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (!_compilerClauseSuccess) CGRuntime::setLocation(_compilerClauseCursor_" << iCursor << ");";
		theCompilerEnvironment.getBody().endl();
	}
}

std::string BNFConjunction::toString() const {
	std::string sText;
	for (std::vector<GrfCommand*>::const_iterator i = getCommands().begin(); i != getCommands().end(); i++) {
		if ((*i)->isABNFCommand()) {
			if (i != getCommands().begin()) sText += " ";
			sText += (*i)->toString();
		}
	}
	return sText;
}
