/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFEndOfFile.h"

using namespace CodeWorker;

BNFEndOfFile::BNFEndOfFile(DtaBNFScript* pBNFScript, GrfBlock* pParent, bool bContinue) : _pBNFScript(pBNFScript), GrfCommand(pParent), _bContinue(bContinue) {}

BNFEndOfFile::~BNFEndOfFile() {
}

bool BNFEndOfFile::isABNFCommand() const { return true; }

SEQUENCE_INTERRUPTION_LIST BNFEndOfFile::executeInternal(DtaScriptVariable& visibility) {
	int iLocation = CGRuntime::getLocation();
	int iImplicitCopyPosition = _pBNFScript->skipEmptyChars(visibility);
	if (!CGRuntime::readChar().empty()) {
		if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
		CGRuntime::setLocation(iLocation);
		if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
		return BREAK_INTERRUPTION;
	}
	return NO_INTERRUPTION;
}

void BNFEndOfFile::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
	theCompilerEnvironment.getBody().endl();
	int iCursor = theCompilerEnvironment.newCursor();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseLocation_" << iCursor << " = CGRuntime::getLocation();";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseImplicitCopy_" << iCursor << " = theEnvironment.skipEmptyChars();";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerClauseSuccess = (CGRuntime::readCharAsInt() < 0);";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (!_compilerClauseSuccess) ";
	if (_bContinue) {
		theCompilerEnvironment.getBody() << "CGRuntime::throwBNFExecutionError(";
		theCompilerEnvironment.getBody().writeString(toString());
		theCompilerEnvironment.getBody() << ");";
	} else {
		theCompilerEnvironment.getBody() << "CGRuntime::setLocation(_compilerClauseLocation_" << iCursor << ");";
	}
	theCompilerEnvironment.getBody().endl();
}

std::string BNFEndOfFile::toString() const {
	std::string sText;
	if (_bContinue) sText = "#continue ";
	sText += "#empty";
	return sText;
}
