/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFNot.h"

using namespace CodeWorker;

BNFNot::BNFNot(DtaBNFScript* pBNFScript, GrfBlock* pParent, bool bContinue) : _pBNFScript(pBNFScript), GrfBlock(pParent), _bContinue(bContinue) {}

BNFNot::~BNFNot() {
}

bool BNFNot::isABNFCommand() const { return true; }

SEQUENCE_INTERRUPTION_LIST BNFNot::executeInternal(DtaScriptVariable& visibility) {
	int iLocation = CGRuntime::getLocation();
	int iImplicitCopyPosition = _pBNFScript->skipEmptyChars(visibility);
	SEQUENCE_INTERRUPTION_LIST result = GrfBlock::executeInternal(visibility);
	if (result == NO_INTERRUPTION) {
		if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
		result = BREAK_INTERRUPTION;
	} else if (result != CONTINUE_INTERRUPTION) {
		result = NO_INTERRUPTION;
	}
	CGRuntime::setLocation(iLocation);
	if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
	return result;
}

void BNFNot::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	std::vector<GrfCommand*>::const_iterator i = getCommands().begin();
	if (i != getCommands().end()) {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
		theCompilerEnvironment.getBody().endl();
		int iCursor = theCompilerEnvironment.newCursor();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseCursor_" << iCursor << " = CGRuntime::getLocation();";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseImplicitCopy_" << iCursor << " = theEnvironment.skipEmptyChars();";
		theCompilerEnvironment.getBody().endl();
		for (;;) {
			(*i)->compileCpp(theCompilerEnvironment);
			i++;
			if (i == getCommands().end()) break;
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
		}
		int j = getNbCommands() - 1;
		while (j > 0) {
			theCompilerEnvironment.decrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
			j--;
		}
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) ";
		if (_bContinue) {
			theCompilerEnvironment.getBody() << "CGRuntime::throwBNFExecutionError(";
			theCompilerEnvironment.getBody().writeString(toString());
			theCompilerEnvironment.getBody() << ");";
		} else {
			theCompilerEnvironment.getBody() << "_compilerClauseSuccess = false;";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "else _compilerClauseSuccess = true;";
		}
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tif (_compilerClauseImplicitCopy_" << iCursor << " >= 0) CGRuntime::resizeOutputStream(_compilerClauseImplicitCopy_" << iCursor << ");";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tCGRuntime::setLocation(_compilerClauseCursor_" << iCursor << ");";
		theCompilerEnvironment.getBody().endl();
	}
}

std::string BNFNot::toString() const {
	std::string sText = "!" + getCommands()[0]->toString();
	if (_bContinue) sText = "#continue " + sText;
	return sText;
}
