/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFPushItem.h"

namespace CodeWorker {

	BNFPushItem::BNFPushItem(DtaBNFScript* pBNFScript, GrfBlock* pParent) : _pBNFScript(pBNFScript), GrfBlock(pParent), _pVariable(0L) {}

	BNFPushItem::~BNFPushItem() {
	}

	bool BNFPushItem::isABNFCommand() const { return true; }

	SEQUENCE_INTERRUPTION_LIST BNFPushItem::execute(DtaScriptVariable& visibility) {
		SEQUENCE_INTERRUPTION_LIST result;
		DtaScriptVariable* pVariable = visibility.getExistingVariable(*_pVariable);
		bool bCreation = (pVariable == 0L);
		if (bCreation) pVariable = visibility.getVariable(*_pVariable);
		pVariable->pushItem("");
		int iCursor = CGRuntime::getLocation();
		int iImplicitCopyPosition = ((_pBNFScript->implicitCopy()) ? CGRuntime::getOutputLocation() : -1);
		result = GrfBlock::executeInternal(visibility);
		if (result != NO_INTERRUPTION) {
			if (bCreation) CGRuntime::removeVariable(pVariable);
			else CGRuntime::removeLastElement(pVariable);
			CGRuntime::setLocation(iCursor);
			if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
		}
		return result;
	}

	void BNFPushItem::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		std::vector<GrfCommand*>::const_iterator i = getCommands().begin();
		if (i != getCommands().end()) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
			theCompilerEnvironment.getBody().endl();
			int iCursor = theCompilerEnvironment.newCursor();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerClauseCursor_" << iCursor << " = CGRuntime::getLocation();";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "bool _compilerClauseCreation_" << iCursor << " = GCRuntime::existVariable(";
			_pVariable->compileCpp(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ");";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation();
			_pVariable->compileCppForSet(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ".pushItem(\"\");";
			theCompilerEnvironment.getBody().endl();
			for (;;) {
				(*i)->compileCpp(theCompilerEnvironment);
				i++;
				if (i == getCommands().end()) break;
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerClauseSuccess) {";
				theCompilerEnvironment.getBody().endl();
				theCompilerEnvironment.incrementIndentation();
			}
			int j = getNbCommands() - 1;
			while (j > 0) {
				theCompilerEnvironment.decrementIndentation();
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
				theCompilerEnvironment.getBody().endl();
				j--;
			}
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (!_compilerClauseSuccess) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tif (_compilerClauseCreation_" << iCursor << ") CGRuntime::removeVariable(";
			_pVariable->compileCppForSet(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ");";
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\telse CGRuntime::removeLastElement(";
			_pVariable->compileCppForSet(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ");";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "\tCGRuntime::setLocation(_compilerClauseCursor_" << iCursor << ");";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
		}
	}

	std::string BNFPushItem::toString() const {
		std::string sText = "#pushItem(" + _pVariable->toString() + ")";
		for (std::vector<GrfCommand*>::const_iterator i = getCommands().begin(); i != getCommands().end(); i++) {
			if ((*i)->isABNFCommand()) {
				if (i != getCommands().begin()) sText += " ";
				sText += (*i)->toString();
			}
		}
		return sText;
	}

}
