/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2003 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFScanWindow.h"

namespace CodeWorker {
	BNFScanWindow::BNFScanWindow(DtaBNFScript* pBNFScript, GrfBlock* pParent, bool bContinue) : _pBNFScript(pBNFScript), GrfBlock(pParent), _bContinue(bContinue), _pWindowSequence(0L) {}

	BNFScanWindow::~BNFScanWindow() {
		delete _pWindowSequence;
	}

	bool BNFScanWindow::isABNFCommand() const { return true; }

	SEQUENCE_INTERRUPTION_LIST BNFScanWindow::executeInternal(DtaScriptVariable& visibility) {
		int iLocation = CGRuntime::getLocation();
		int iImplicitCopyPosition = _pBNFScript->skipEmptyChars(visibility);
		SEQUENCE_INTERRUPTION_LIST result = _pWindowSequence->execute(visibility);
		if (result != CONTINUE_INTERRUPTION) {
			int iFinalLocation = CGRuntime::getLocation();
			CGRuntime::setLocation(iLocation);
			if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
			if (iFinalLocation < iLocation) result = BREAK_INTERRUPTION;
			if (result == NO_INTERRUPTION) {
				ScpStream::SizeAttributes sizeAttrs = CGRuntime::_pInputStream->resize(iFinalLocation);
				result = GrfBlock::executeInternal(visibility);
				CGRuntime::_pInputStream->restoreSize(sizeAttrs);
				if ((result == NO_INTERRUPTION) || (result == CONTINUE_INTERRUPTION)) {
					CGRuntime::setLocation(iFinalLocation);
				} else {
					CGRuntime::setLocation(iLocation);
					if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
					if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
				}
			} else {
				if (iFinalLocation < iLocation) throw UtlException("Left member of '" + CGRuntime::composeCLikeString(toString()) + "' shouldn't set the cursor back in the sentence");
				if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
			}
		}
		return result;
	}

	void BNFScanWindow::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "<unhandled \"BNFScanWindow.cpp\">";
		theCompilerEnvironment.getBody().endl();
	}

	std::string BNFScanWindow::toString() const {
		std::string sText = "[" + _pWindowSequence->toString() + "] |> [" + getCommands()[0]->toString() + "]";
		if (_bContinue) sText = "#continue " + sText;
		return sText;
	}
}
