/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2003 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "BNFClause.h"
#include "BNFSkipIgnore.h"

using namespace CodeWorker;

BNFSkipIgnore::BNFSkipIgnore(DtaBNFScript* pBNFScript, GrfBlock* pParent, bool bContinue) : _pBNFScript(pBNFScript), GrfCommand(pParent), _iClauseReturnType(BNFClause::NO_RETURN_TYPE), _bContinue(bContinue), _pVariableToAssign(0L) {}
BNFSkipIgnore::~BNFSkipIgnore() {}

bool BNFSkipIgnore::isABNFCommand() const { return true; }

void BNFSkipIgnore::setVariableToAssign(ExprScriptVariable* pVariableToAssign, bool bConcat, BNFClause& theClause) {
	if (pVariableToAssign != 0L) {
		_pVariableToAssign = pVariableToAssign;
		std::string sVariableName = _pVariableToAssign->toString();
		if (sVariableName == theClause.getName()) _iClauseReturnType = theClause.getReturnType();
		_bConcatVariable = bConcat;
	}
}

SEQUENCE_INTERRUPTION_LIST BNFSkipIgnore::executeInternal(DtaScriptVariable& visibility) {
	int iLocation = CGRuntime::getLocation();
	int iImplicitCopyPosition = _pBNFScript->skipEmptyChars(visibility);
	if (!_listOfConstants.empty()) {
		std::string sValue = CGRuntime::getLastReadChars(CGRuntime::getLocation() - iLocation);
		bool bMatch = false;
		for (std::vector<std::string>::const_iterator i = _listOfConstants.begin(); i != _listOfConstants.end(); i++) {
			bMatch = (*i == sValue);
			if (bMatch) break;
		}
		if (!bMatch) {
			if (_bContinue) CGRuntime::throwBNFExecutionError(toString());
			CGRuntime::setLocation(iLocation);
			if (iImplicitCopyPosition >= 0) CGRuntime::resizeOutputStream(iImplicitCopyPosition);
			return BREAK_INTERRUPTION;
		}
	}
	if (_pVariableToAssign != 0L) {
		std::string sValue = CGRuntime::getLastReadChars(CGRuntime::getLocation() - iLocation);
		DtaScriptVariable* pVariable = visibility.getVariable(*_pVariableToAssign);
		if (_iClauseReturnType == BNFClause::LIST_RETURN_TYPE) {
			pVariable->pushItem(sValue);
		} else {
			if (_bConcatVariable) pVariable->concatValue(sValue.c_str());
			else pVariable->setValue(sValue.c_str());
		}
	}
	return NO_INTERRUPTION;
}

void BNFSkipIgnore::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	int iCursor = theCompilerEnvironment.newCursor();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "// " << toString();
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "<#skipIgnore not implemented yet!>";
	theCompilerEnvironment.getBody().endl();
}

std::string BNFSkipIgnore::toString() const {
	std::string sString;
	if (_bContinue) sString = "#continue ";
	sString += "#skipIgnore";
	return sString + DtaBNFScript::assignmentToString(_pVariableToAssign);
}
