/*
 * CppParsingTree.h
 *
 * Copyright (C) 1996-1997, 1999-2002 Cedric Lemaire
 *
 */
#ifndef _CppParsingTree_h_
#define _CppParsingTree_h_

#include <string>
#include <list>
#include <vector>
#include <map>
#include <memory>

namespace CodeWorker {
	class DtaScriptVariable;
	class ExprScriptExpression;
	class ExprScriptVariable;
	class CGRuntime;


	class CppParsingTree_expr {
		protected:
			ExprScriptExpression* _pExpression;

		public:
			inline CppParsingTree_expr() : _pExpression(0L) {}
			CppParsingTree_expr(const std::string& sExpression);
			~CppParsingTree_expr();

			inline ExprScriptExpression& getExpression() const { return *_pExpression; }
	};

	class CppParsingTree_varexpr : public CppParsingTree_expr {
		public:
			CppParsingTree_varexpr(const std::string& sVariableExpression);

			ExprScriptVariable& getVariableExpression() const;
	};


	class CppParsingTree_var {
		protected:
			DtaScriptVariable* _pInternalNode;
			friend class CGRuntime;

			CppParsingTree_var();

		public:
			CppParsingTree_var(const CppParsingTree_var& tree);
			CppParsingTree_var(DtaScriptVariable* pVariable);
			CppParsingTree_var(DtaScriptVariable& variable);

			inline bool isNull() const { return (_pInternalNode == 0L); }
			const char* getName() const;
			const std::string& getValue() const;
			bool getBooleanValue() const;
			int getIntValue() const;
			double getDoubleValue() const;

			void setValue(const char* sValue);
			void setValue(const std::string& sValue);
			void setValue(bool bValue);
			void setValue(double dValue);
			void setValue(int iValue);
			void setValue(const CppParsingTree_var& theValue);
			void setValue(const std::list<std::string>& listOfValues);
			void concatenateValue(const std::string& sValue);
			void concatenateValue(const CppParsingTree_var& pValue);
			void setReference(CppParsingTree_var& reference);
			void setAll(const CppParsingTree_var& tree);
			void merge(const CppParsingTree_var& tree);
			void pushItem(const std::string& sValue);
			void addElements(const CppParsingTree_var& theValue);
			const std::list<DtaScriptVariable*>& getArray() const;
			const std::map<std::string, DtaScriptVariable*>& getSortedArray() const;
			std::auto_ptr<std::vector<DtaScriptVariable*> > getSortedNoCaseArray() const;

			void clearNode();

			CppParsingTree_var getNode(const std::string& sBranch) const;
			CppParsingTree_var getParentNode() const;
			CppParsingTree_var getArrayNodeFromKey(const std::string& sKey) const;
			CppParsingTree_var getArrayNodeFromPosition(int iPosition) const;
			CppParsingTree_var getFirstArrayNode() const;
			CppParsingTree_var getLastArrayNode() const;

			CppParsingTree_var insertNode(const std::string& sBranch);
			CppParsingTree_var insertArrayNodeFromKey(const std::string& sKey);
			CppParsingTree_var insertArrayNodeFromKey(int iKey);
			CppParsingTree_var insertArrayNodeFromKey(const CppParsingTree_var& pKey);
	};

	class CppParsingTree_value : public CppParsingTree_var {
		public:
			CppParsingTree_value();
			CppParsingTree_value(bool bValue);
			CppParsingTree_value(int iValue);
			CppParsingTree_value(const char* tcValue);
			CppParsingTree_value(const std::string& sValue);
			CppParsingTree_value(const CppParsingTree_var& pTree);
			CppParsingTree_value(const CppParsingTree_value& pTree);
			~CppParsingTree_value();
	};
}

#endif
