/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#pragma warning (disable : 4503)
#endif

#ifndef WIN32
#	include <cstdio> // for Debian/gcc 2.95.4
#endif

#include "ScpStream.h"

#include "UtlException.h"
#include "UtlTrace.h"
#include "UtlString.h"

#include "ExprScriptVariable.h"
#include "Workspace.h"
#include "DtaProject.h"
#include "CGRuntime.h"
#include "DtaScriptVariable.h"

using namespace CodeWorker;


void DtaScriptVariableList::pushBack(DtaScriptVariable* pVariable) {
	register DtaScriptVariableList* pList = this;
	while (pList->getNext() != 0L) pList = pList->getNext();
	pList->setNext(new DtaScriptVariableList(pVariable));
}


DtaScriptVariable::~DtaScriptVariable() {
	clearContent();
	if (_pReferencedByVariables != 0L) {
		while (!_pReferencedByVariables->empty()) {
			DtaScriptVariable* pReferencedBy = *(_pReferencedByVariables->begin());
			pReferencedBy->removeReference();
/*
			std::string sErrorMessage = "warning: variable '";
			sErrorMessage += pReferencedBy->getCompleteName();
			sErrorMessage += "' refers to '";
			sErrorMessage += getCompleteName();
			sErrorMessage += "' that is being removed from the stack" + CGRuntime::endl();
			sErrorMessage += UtlTrace::getTraceStack();
			if (DtaProject::getInstance().addWarning(sErrorMessage) == 1) CGRuntime::traceLine(sErrorMessage);
*/
		}
		delete _pReferencedByVariables;
		_pReferencedByVariables = 0L;
	}
	free((char*) _tcName);
}

bool DtaScriptVariable::isIndex() const { return false; }

const std::list<DtaScriptVariable*>& DtaScriptVariable::getArray() const {
	const DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	return pVariable->_array;
}

int DtaScriptVariable::getArraySize() const {
	const DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	return pVariable->_array.size();
}

const std::map<std::string, DtaScriptVariable*>& DtaScriptVariable::getSortedArray() const {
	const DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	if (pVariable->_pArrayCache == 0L) {
		pVariable->_pArrayCache = new std::map<std::string, DtaScriptVariable*>();
		for (std::list<DtaScriptVariable*>::const_iterator i = pVariable->_array.begin(); i != pVariable->_array.end(); i++) {
			(*(pVariable->_pArrayCache))[(*i)->_tcName] = (*i);
		}
	}
	return *(pVariable->_pArrayCache);
}

std::auto_ptr<std::vector<DtaScriptVariable*> > DtaScriptVariable::getSortedNoCaseArray() const {
	const DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	std::map<std::string, std::map<std::string, DtaScriptVariable*> > mapOfNodes;
	for (std::list<DtaScriptVariable*>::const_iterator i = pVariable->_array.begin(); i != pVariable->_array.end(); i++) {
		std::string sKey = toLowercase((*i)->_tcName);
		mapOfNodes[sKey][(*i)->_tcName] = *i;
	}
	std::vector<DtaScriptVariable*>* pListOfNodes = new std::vector<DtaScriptVariable*>(pVariable->_array.size());
	int iPosition = 0;
	for (std::map<std::string, std::map<std::string, DtaScriptVariable*> >::const_iterator j = mapOfNodes.begin(); j != mapOfNodes.end(); ++j) {
		for (std::map<std::string, DtaScriptVariable*>::const_iterator k = j->second.begin(); k != j->second.end(); ++k) {
			(*pListOfNodes)[iPosition++] = k->second;
		}
	}
	return std::auto_ptr<std::vector<DtaScriptVariable*> >(pListOfNodes);
}

DtaScriptVariable* DtaScriptVariable::getArrayElement(const std::string& sPosition) const {
	if (getReferencedVariable() != 0L) return getReferencedVariable()->getArrayElement(sPosition);
	if (_pArrayCache != 0L) {
		std::map<std::string, DtaScriptVariable*>::const_iterator i = _pArrayCache->find(sPosition);
		if (i != _pArrayCache->end()) return i->second;
	} else if (_array.size() >= 12) {
		DtaScriptVariable* pVariable = 0L;
		_pArrayCache = new std::map<std::string, DtaScriptVariable*>();
		for (std::list<DtaScriptVariable*>::const_iterator i = _array.begin(); i != _array.end(); i++) {
			if (strcmp((*i)->_tcName, sPosition.c_str()) == 0) pVariable = (*i);
			(*_pArrayCache)[(*i)->_tcName] = (*i);
		}
		return pVariable;
	} else {
		for (std::list<DtaScriptVariable*>::const_iterator i = _array.begin(); i != _array.end(); i++) {
			if (strcmp((*i)->_tcName, sPosition.c_str()) == 0) return (*i);
		}
	}
	return 0L;
}

DtaScriptVariable* DtaScriptVariable::getArrayElement(int iPosition) const {
	if (getReferencedVariable() != 0L) return getReferencedVariable()->getArrayElement(iPosition);
	if (iPosition >= 0) {
		for (std::list<DtaScriptVariable*>::const_iterator i = _array.begin(); i != _array.end(); i++) {
			if (iPosition == 0) return (*i);
			iPosition--;
		}
	}
	return 0L;
}

void DtaScriptVariable::invertArray() {
	if (getReferencedVariable() != 0L) {
		getReferencedVariable()->invertArray();
	} else {
		std::list<DtaScriptVariable*> array;
		for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
			array.push_front(*i);
		}
		_array = array;
	}
}

void DtaScriptVariable::removeArrayElement() {
	if (getReferencedVariable() != 0L) {
		getReferencedVariable()->removeArrayElement();
	} else {
		for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
			delete (*i);
		}
		if (_pArrayCache != 0L) {
			delete _pArrayCache;
			_pArrayCache = 0L;
		}
		_array = std::list<DtaScriptVariable*>();
	}
}

void DtaScriptVariable::removeArrayElement(const char* sPosition) {
	if (getReferencedVariable() != 0L) getReferencedVariable()->removeArrayElement(sPosition);
	else {
		for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
			if (strcmp((*i)->_tcName, sPosition) == 0) {
				delete (*i);
				_array.erase(i);
				if (_pArrayCache != 0L) _pArrayCache->erase(sPosition);
				break;
			}
		}
	}
}

void DtaScriptVariable::removeArrayElement(int iPosition) {
	if (getReferencedVariable() != 0L) getReferencedVariable()->removeArrayElement(iPosition);
	else {
		if (iPosition >= 0) {
			for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
				if (iPosition == 0) {
					if (_pArrayCache != 0L) _pArrayCache->erase((*i)->_tcName);
					delete *i;
					_array.erase(i);
					break;
				}
				iPosition--;
			}
		}
	}
}

void DtaScriptVariable::removeFirstElement() {
	if (getReferencedVariable() != 0L) getReferencedVariable()->removeFirstElement();
	else {
		std::list<DtaScriptVariable*>::iterator cursor = _array.begin();
		if (cursor != _array.end()) {
			if (_pArrayCache != 0L) _pArrayCache->erase((*cursor)->_tcName);
			delete *cursor;
			_array.erase(cursor);
		}
	}
}

void DtaScriptVariable::removeLastElement() {
	if (getReferencedVariable() != 0L) getReferencedVariable()->removeLastElement();
	else if (!_array.empty()) {
		DtaScriptVariable* pElt = _array.back();
		if (_pArrayCache != 0L) _pArrayCache->erase(pElt->_tcName);
		_array.remove(pElt);
		delete pElt;
	}
}

std::string DtaScriptVariable::getCompleteName() const {
	std::string sName;
	if (_pParent != 0L) {
		sName = _pParent->getCompleteName();
		DtaScriptVariableList* pAttributes = _pParent->_pAttributes;
		while (pAttributes != 0L) {
		    if (strcmp(pAttributes->getNode()->_tcName, _tcName) == 0) {
				if (!sName.empty()) sName += ".";
				sName += _tcName;
				return sName;
			}
			pAttributes = pAttributes->getNext();
		}
		for (std::list<DtaScriptVariable*>::const_iterator i = _pParent->_array.begin(); i != _pParent->_array.end(); i++) {
			if (((*i) != 0L) && (strcmp((*i)->_tcName, _tcName) == 0)) {
				sName += "[\"" + std::string(_tcName) + "\"]";
				return sName;
			}
		}
		sName = "";
	} else sName = _tcName;
	return sName;
}

void DtaScriptVariable::clearArray() {
	for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
		delete (*i);
		*i = 0L;
	}
	_array = std::list<DtaScriptVariable*>();
	delete _pArrayCache;
	_pArrayCache = 0L;
}

void DtaScriptVariable::removeReference() {
	_pVariableValue->_pReferencedByVariables->erase(this);
	_pVariableValue = 0L;
}

void DtaScriptVariable::addReferencedBy(DtaScriptVariable* pVariable) {
	if (pVariable != 0L) {
		if (_pReferencedByVariables == 0L) _pReferencedByVariables = new std::set<DtaScriptVariable*>();
		_pReferencedByVariables->insert(pVariable);
	}
}

DtaScriptVariable* DtaScriptVariable::getInternalVariableExtraction(DtaScriptVariable& visibility, const ExprScriptVariable& exprVariable, bool bRecursiveSearch, bool bCreateIfUnknown, bool bReferencedVariable) {
    DtaScriptVariable* pVariable = 0L;
	if (exprVariable.isEvaluation()) {
		std::string sCommand = exprVariable.getExpression()->getValue(visibility);
		ScpStream theCommand(sCommand);
		DtaScript script;
		std::auto_ptr<ExprScriptVariable> pExprVariable(script.parseVariableExpression(script.getBlock(), theCommand));
		pVariable = getInternalVariableExtraction(visibility, *pExprVariable, bRecursiveSearch, bCreateIfUnknown, bReferencedVariable);
		return pVariable;
	}
	if (bReferencedVariable || exprVariable.isNext()) pVariable = getReferencedVariable();
	if (pVariable == 0L) {
		bool bExtractVariableWithReference = bReferencedVariable || !exprVariable.isAttributeOnly();
		std::string sName = exprVariable.getName();
		pVariable = getVariable(sName.c_str(), bRecursiveSearch, bCreateIfUnknown, bExtractVariableWithReference);
		if (pVariable == 0L) return 0L;
	}
	if (exprVariable.isArrayKey()) {
		std::string sPosition = exprVariable.getExpression()->getValue(visibility);
		DtaScriptVariable* pArrayElement = pVariable->getArrayElement(sPosition);
		if (pArrayElement == 0L) {
			if (bCreateIfUnknown) {
				pArrayElement = pVariable->addInternalElement(sPosition);
			} else {
				return 0L;
			}
		    pVariable = pArrayElement;
		} else {
			if (bReferencedVariable || exprVariable.isNext()) {
				pVariable = pArrayElement->getReferencedVariable();
				if (pVariable == 0L) pVariable = pArrayElement;
			} else {
			    pVariable = pArrayElement;
			}
		}
	} else if (exprVariable.isArrayPosition()) {
		std::string sPosition = exprVariable.getExpression()->getValue(visibility);
		if (sPosition == "parent") {
			register DtaScriptVariable* pParent = pVariable->getParent();
			if ((pParent != 0L) && (strncmp(pParent->_tcName, "##stack##", 9) != 0)) pVariable = pParent;
		} else {
			int iPosition;
			if (sPosition == "back") iPosition = pVariable->getArraySize() - 1;
			else iPosition = atoi(sPosition.c_str());
			DtaScriptVariable* pArrayElement = pVariable->getArrayElement(iPosition);
			if (pArrayElement == 0L) throw UtlException(std::string("Array index is out of bounds on variable '" + exprVariable.toString() + "': \"") + sPosition + "\"");
			pVariable = pArrayElement;
			if (bReferencedVariable || exprVariable.isNext()) {
				pVariable = pVariable->getReferencedVariable();
				if (pVariable == 0L) pVariable = pArrayElement;
			}
		}
	}
    if (exprVariable.isNext()) {
		pVariable = pVariable->getInternalVariableExtraction(visibility, *(exprVariable.getNext()), false, bCreateIfUnknown, bReferencedVariable);
	}
	return pVariable;
}

DtaScriptVariable* DtaScriptVariable::getVariable(const ExprScriptVariable& exprVariable) {
	return getInternalVariableExtraction(*this, exprVariable, true, true, true);
}

DtaScriptVariable* DtaScriptVariable::getNonRecursiveVariable(DtaScriptVariable& visibility, const ExprScriptVariable& exprVariable) {
	return getInternalVariableExtraction(visibility, exprVariable, false, true, true);
}

DtaScriptVariable* DtaScriptVariable::getVariableForReferenceAssignment(const ExprScriptVariable& exprVariable) {
	return getInternalVariableExtraction(*this, exprVariable, true, true, false);
}

DtaScriptVariable* DtaScriptVariable::getExistingVariable(const ExprScriptVariable& exprVariable) {
	DtaScriptVariable* pVariable = getInternalVariableExtraction(*this, exprVariable, true, false, true);
	if ((pVariable == 0L) && Workspace::warningOnUnknownVariables()) {
		std::string sErrorMessage = "warning: variable '" + exprVariable.toString() + "' doesn't exist yet" + CGRuntime::endl();
		sErrorMessage += UtlTrace::getTraceStack();
		if (DtaProject::getInstance().addWarning(sErrorMessage) == 1) CGRuntime::traceLine(sErrorMessage);
	}
	return pVariable;
}

DtaScriptVariable* DtaScriptVariable::getNoWarningExistingVariable(const ExprScriptVariable& exprVariable) {
	return getInternalVariableExtraction(*this, exprVariable, true, false, true);
}


DtaScriptVariable* DtaScriptVariable::getNonRecursiveVariable(const char* sName) {
	return getVariable(sName, false, true, true);
}

DtaScriptVariable* DtaScriptVariable::getVariable(const char* sName, bool bRecursiveSearch, bool bCreateIfUnknown, bool bReferencedVariable) {
    DtaScriptVariable* pVariable = 0L;
	if (*sName == '\0') return this;
	if (bRecursiveSearch) {
		if (strcmp(sName, "project") == 0) return &(DtaProject::getInstance());
		if (strcmp(sName, "this") == 0) {
			DtaScriptVariable* pThis = this;
			while ((pThis->_tcName != 0L) && (strncmp(pThis->_tcName, "##stack##", 9) == 0)) pThis = pThis->getParent();
			if (pThis != &CGRuntime::getThisInternalNode()) {
				if (&CGRuntime::getThisInternalNode() == 0L) throw UtlException("NULL 'this'!!!!!");
				throw UtlException("internal error: 2 different 'this' (" + pThis->getCompleteName() + " and " + CGRuntime::getThisInternalNode().getCompleteName() + ")!");
			}
			return pThis;
		}
	}
	DtaScriptVariableList* pAttributes = _pAttributes;
	DtaScriptVariableList* pLastAttribute = _pAttributes;
    while (pAttributes != 0L) {
	    if (strcmp(pAttributes->getNode()->_tcName, sName) == 0) {
            pVariable = pAttributes->getNode();
		    if (bReferencedVariable && (pVariable->getReferencedVariable() != 0L)) pVariable = pVariable->getReferencedVariable();
		    break;
		}
		if (pAttributes->getNext() == 0L) pLastAttribute = pAttributes;
		pAttributes = pAttributes->getNext();
	}
    if (pAttributes == 0L) {
		if ((_pParent != 0L) && bRecursiveSearch) {
//			pVariable = _pParent->getVariable(sName, true, false, bReferencedVariable);
			// we are looking for the variable into local variables, above the function call
			DtaScriptVariable* pLocalScope = 0L;
			bool bClause = false;
			if ((strncmp(_tcName, "##stack## ", 10) == 0) && (strncmp(_tcName + 10, "function", 8) != 0) && (strncmp(_tcName + 10, "clause", 6) != 0)) {
				pLocalScope = getParent();
				while ((pVariable == 0L) && (strncmp(pLocalScope->_tcName, "##stack##", 9) == 0)) {
					pVariable = pLocalScope->getVariable(sName, false, false, bReferencedVariable);
					const char* tcFunction = pLocalScope->_tcName + 10;
					if (strncmp(tcFunction, "function", 8) == 0) break;
					if (strncmp(tcFunction, "clause", 6) == 0) {
						bClause = true;
						break;
					}
					pLocalScope = pLocalScope->getParent();
				}
			}
			if ((pVariable == 0L) && !bClause) {
				// the variable doesn't belong to the stack, perhaps it is a node of the current subtree
				DtaScriptVariable* pThis = getVariable("this");
				pVariable = pThis->getVariable(sName, false, false, bReferencedVariable);
				int tiVersion[] = {1, 13, -1};
				if ((pVariable == 0L) && (pLocalScope != 0L) && DtaProject::getInstance().isScriptVersionOlderThan(tiVersion)) {
					// this is an old version of scripts, so variable may be stored
					// into the stack, above the function call
					while ((pVariable == 0L) && (strncmp(pLocalScope->_tcName, "##stack##", 9) == 0)) {
						pVariable = pLocalScope->getVariable(sName, false, false, bReferencedVariable);
						pLocalScope = pLocalScope->getParent();
					}
					if (pVariable != 0L) {
						std::string sErrorMessage = "since version 1.13, the scope of a local variable is restricted to the function it is declared into, so a change of behaviour may occur when looking at local variable '";
						sErrorMessage += sName;
						sErrorMessage += "'" + UtlTrace::getTraceStack();
						if (DtaProject::getInstance().addWarning(sErrorMessage) == 1) CGRuntime::traceLine(sErrorMessage);
						pVariable = 0L;
					}
				}
			}
		}
		if (pVariable == 0L) {
			if (bRecursiveSearch) pVariable = DtaProject::getInstance().getGlobalVariable(sName);
			if ((pVariable == 0L) && bCreateIfUnknown) {
				pVariable = new DtaScriptVariable(this, sName);
				if (_pAttributes == 0L) _pAttributes = new DtaScriptVariableList(pVariable);
				else pLastAttribute->pushBack(pVariable);
			}
		}
    }
	return pVariable;
}

DtaScriptVariable* DtaScriptVariable::getNode(const char* sName) {
	DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	return pVariable->getVariable(sName, false, false, true);
}

DtaScriptVariable* DtaScriptVariable::insertNode(const char* sName) {
	DtaScriptVariable* pVariable = this;
	while (pVariable->getReferencedVariable() != 0L) pVariable = pVariable->getReferencedVariable();
	return pVariable->getVariable(sName, false, true, true);
}

DtaScriptIndex* DtaScriptVariable::createIndex(const char* sName) {
    DtaScriptVariable* pIndex = 0L;
    DtaScriptVariableList* pAttributes = _pAttributes;
    DtaScriptVariableList* pLastAttribute = _pAttributes;
    while (pAttributes != 0L) {
	    if (strcmp(pAttributes->getNode()->_tcName, sName) == 0) {
            pIndex = pAttributes->getNode();
		    break;
		}
		if (pAttributes->getNext() == 0L) pLastAttribute = pAttributes;
		pAttributes = pAttributes->getNext();
	}
    if (pAttributes == 0L) {
	    pIndex = new DtaScriptIndex(this, sName);
		if (_pAttributes == 0L) _pAttributes = new DtaScriptVariableList(pIndex);
		else pLastAttribute->pushBack(pIndex);
    }
	if (!pIndex->isIndex()) return 0L;
	return (DtaScriptIndex*) pIndex;
}

DtaScriptIndex* DtaScriptVariable::getIndex(const char* sName) const {
    DtaScriptVariable* pIndex = 0L;
    DtaScriptVariableList* pAttributes = _pAttributes;
    DtaScriptVariableList* pLastAttribute = _pAttributes;
    while (pAttributes != 0L) {
	    if (strcmp(pAttributes->getNode()->_tcName, sName) == 0) {
            pIndex = pAttributes->getNode();
		    break;
		}
		if (pAttributes->getNext() == 0L) pLastAttribute = pAttributes;
		pAttributes = pAttributes->getNext();
	}
    if (pAttributes == 0L) {
		if (_pParent != 0L) pIndex = _pParent->getIndex(sName);
		if (pIndex == 0L) {
			throw UtlException(std::string("unable to find index called '") + sName + "'");
		}
    }
	if (!pIndex->isIndex()) throw UtlException(std::string("variable called '") + sName + "' isn't an index");
	return (DtaScriptIndex*) pIndex;
}

DtaScriptVariable* DtaScriptVariable::addElement(const std::string& sName) {
    DtaScriptVariable* pVariable = getArrayElement(sName);
	if (pVariable == 0L) pVariable = addInternalElement(sName);
	return pVariable;
}

DtaScriptVariable* DtaScriptVariable::addInternalElement(const std::string& sName) {
	DtaScriptVariable* pVariable = new DtaScriptVariable(this, sName);
	_array.push_back(pVariable);
	if (_pArrayCache != 0L) (*_pArrayCache)[sName] = pVariable;
	return pVariable;
}

DtaScriptVariable* DtaScriptVariable::addElement(DtaScriptVariable* pVariable) {
	for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
	    if (strcmp((*i)->_tcName, pVariable->_tcName) == 0) {
			if ((*i) == pVariable) return pVariable;
			_array.erase(i);
			break;
		}
	}
	_array.push_back(pVariable);
	if (_pArrayCache != 0L) (*_pArrayCache)[pVariable->_tcName] = pVariable;
	return pVariable;
}

void DtaScriptVariable::setValueAtVariable(const char* sVariable, const char* sValue, bool bRecursiveSearch, bool bCreateIfUnknown) {
	getVariable(sVariable, bRecursiveSearch, bCreateIfUnknown)->setValue(sValue);
}

const std::string& DtaScriptVariable::getValue() const {
    if (_pVariableValue == 0L) return _sConstantValue;
    return _pVariableValue->getValue();
}

double DtaScriptVariable::getDoubleValue() const {
    if (_pVariableValue != 0L) return _pVariableValue->getDoubleValue();
	return atof(_sConstantValue.c_str());
}

bool DtaScriptVariable::getBooleanValue() const {
    if (_pVariableValue != 0L) return _pVariableValue->getBooleanValue();
	return !_sConstantValue.empty();
}

int DtaScriptVariable::getIntValue() const {
    if (_pVariableValue != 0L) return _pVariableValue->getIntValue();
	return atoi(_sConstantValue.c_str());
}

void DtaScriptVariable::setValue(const char* sValue) {
    if (_pVariableValue != 0L) removeReference();
    _sConstantValue = sValue;
}

void DtaScriptVariable::setValue(bool bValue) {
    if (_pVariableValue != 0L) removeReference();
	if (bValue) _sConstantValue = "true";
	else _sConstantValue = "";
}

void DtaScriptVariable::setValue(double dValue) {
    if (_pVariableValue != 0L) removeReference();
	_sConstantValue = CGRuntime::convertDoubleToString(dValue);
}

void DtaScriptVariable::setValue(int iValue) {
	char tcNumber[16];
	sprintf(tcNumber, "%d", iValue);
	setValue(tcNumber);
}

void DtaScriptVariable::setValue(const std::list<std::string>& listOfValues) {
    int iIndex = 0;
    char sNumber[16];
	clearArray();
	for (std::list<std::string>::const_iterator i = listOfValues.begin(); i != listOfValues.end(); i++) {
        sprintf(sNumber, "%d", iIndex);
		addElement(sNumber)->setValue(i->c_str());
        iIndex++;
	}
}

void DtaScriptVariable::setValue(DtaScriptVariable* pVariable) {
	if (pVariable != 0L) {
		if (pVariable == this) throw UtlException("DtaScriptVariable::setValue() cyclic reference");
		while (pVariable->getReferencedVariable() != 0L) {
			pVariable = pVariable->getReferencedVariable();
			if (pVariable == this) throw UtlException("DtaScriptVariable::setValue() cyclic reference");
		}
	}
	if (_pVariableValue != 0L) removeReference();
	_pVariableValue = pVariable;
	_pVariableValue->addReferencedBy(this);
	_sConstantValue = "";
}

void DtaScriptVariable::concatValue(const char* tcValue) {
    if (_pVariableValue != 0L) {
		_sConstantValue = _pVariableValue->getValue();
		removeReference();
	}
    _sConstantValue += tcValue;
}

void DtaScriptVariable::remove(DtaScriptVariable* pVariable) {
	if (getReferencedVariable() != 0L) getReferencedVariable()->remove(pVariable);
	else {
		if (_pAttributes != 0L) {
			if (_pAttributes->getNode() == pVariable) {
				DtaScriptVariableList* pNext = _pAttributes->getNext();
				delete _pAttributes->getNode();
				_pAttributes->setNext(0L);
				delete _pAttributes;
				_pAttributes = pNext;
				return;
			} else {
				DtaScriptVariableList* pAttributes = _pAttributes;
				DtaScriptVariableList* pNext = pAttributes->getNext();
				while (pNext != 0L) {
					if (pNext->getNode() == pVariable) {
						pAttributes->setNext(pNext->getNext());
						delete pNext->getNode();
						pNext->setNext(0L);
						delete pNext;
						return;
					}
					pAttributes = pNext;
					pNext = pNext->getNext();
				}
			}
		}
		for (std::list<DtaScriptVariable*>::iterator i = _array.begin(); i != _array.end(); i++) {
			if ((*i) == pVariable) {
				std::string sPosition = pVariable->getName();
				delete (*i);
				_array.erase(i);
				if (_pArrayCache != 0L) _pArrayCache->erase(sPosition);
				return;
			}
		}
	}
}

void DtaScriptVariable::remove(const char* sAttribute) {
	if (_pAttributes != 0L) {
		if (strcmp(_pAttributes->getNode()->_tcName, sAttribute) == 0) {
			DtaScriptVariableList* pNext = _pAttributes->getNext();
			delete _pAttributes->getNode();
			_pAttributes->setNext(0L);
			delete _pAttributes;
			_pAttributes = pNext;
		} else {
		    DtaScriptVariableList* pAttributes = _pAttributes;
			DtaScriptVariableList* pNext = pAttributes->getNext();
			while (pNext != 0L) {
				if (strcmp(pNext->getNode()->_tcName, sAttribute) == 0) {
					pAttributes->setNext(pNext->getNext());
					delete pNext->getNode();
					pNext->setNext(0L);
					delete pNext;
					break;
				}
				pAttributes = pNext;
				pNext = pNext->getNext();
			}
		}
	}
}

void DtaScriptVariable::removeRecursive(const char* sAttribute) {
	remove(sAttribute);
	if (_pAttributes != 0L) {
		DtaScriptVariableList* pAttributes = _pAttributes;
		while (pAttributes != 0L) {
			pAttributes->getNode()->removeRecursive(sAttribute);
			pAttributes = pAttributes->getNext();
		}
	}
	for (std::list<DtaScriptVariable*>::iterator j = _array.begin(); j != _array.end(); j++) {
		if ((*j)->getParent() == this) (*j)->removeRecursive(sAttribute);
	}
}

void DtaScriptVariable::clearContent() {
	if (_pVariableValue != 0L) removeReference();
	_sConstantValue = "";
	DtaScriptVariableList* pAttributes = _pAttributes;
    while (pAttributes != 0L) {
		DtaScriptVariableList* pCurrent = pAttributes;
		delete pCurrent->getNode();
		pAttributes = pCurrent->getNext();
		pCurrent->setNext(0L);
		delete pCurrent;
	}
	_pAttributes = 0L;
	clearArray();
}

void DtaScriptVariable::addArrayElements(const DtaScriptVariable& variable) {
    for (std::list<DtaScriptVariable*>::const_iterator j = variable._array.begin(); j != variable._array.end(); j++) {
		DtaScriptVariable* pVariable = new DtaScriptVariable(this, (*j)->_tcName);
		pVariable->copyAll(*(*j), false);
		pVariable->setValue((*j)->getValue().c_str());
		_array.push_back(pVariable);
		if (_pArrayCache != 0L) (*_pArrayCache)[pVariable->_tcName] = pVariable;
	}
}

bool DtaScriptVariable::equal(const DtaScriptVariable& variable) {
	if (_array.size() != variable._array.size()) return false;
	if (_pVariableValue != variable._pVariableValue) return false;
	if (_sConstantValue != variable._sConstantValue) return false;
	DtaScriptVariableList* pAttributes = _pAttributes;
    while (pAttributes != 0L) {
		DtaScriptVariableList* pVariableAttributes = variable._pAttributes;
		DtaScriptVariable* pVariable = pAttributes->getNode();
	    while (pVariableAttributes != 0L) {
			if (strcmp(pVariable->_tcName, pVariableAttributes->getNode()->_tcName) == 0) {
				if (!pVariable->equal(*(pVariableAttributes->getNode()))) return false;
				break;
			}
			pVariableAttributes = pVariableAttributes->getNext();
		}
		if (pVariableAttributes == 0L) return false;
		pAttributes = pAttributes->getNext();
	}
	std::list<DtaScriptVariable*>::iterator i;
	std::list<DtaScriptVariable*>::const_iterator j;
    for (i = _array.begin(); i != _array.end(); i++) {
		for (j = variable._array.begin(); j != variable._array.end(); ++j) {
			if (strcmp((*i)->_tcName, (*j)->_tcName) == 0) {
				if (!(*i)->equal(*(*j))) return false;
				break;
			}
		}
		if (j == variable._array.end()) return false;
	}
	return true;
}

void DtaScriptVariable::copyAll(const DtaScriptVariable& variable, bool bMerge) {
	if (!bMerge) clearContent();
	if (!variable.getValue().empty()) setValue(variable.getValue().c_str());
	DtaScriptVariableList* pVariableAttributes = variable._pAttributes;
    while (pVariableAttributes != 0L) {
		const char* tcName = pVariableAttributes->getNode()->_tcName;
		DtaScriptVariable* pVariable = 0L;
		DtaScriptVariableList* pAttributes = _pAttributes;
		DtaScriptVariableList* pLastAttribute = 0L;
		while (pAttributes != 0L) {
			if (strcmp(pAttributes->getNode()->_tcName, tcName) == 0) {
				pVariable = pAttributes->getNode();
				break;
			}
			if (pAttributes->getNext() == 0L) pLastAttribute = pAttributes;
			pAttributes = pAttributes->getNext();
		}
		if (pVariable == 0L) {
			pVariable = new DtaScriptVariable(this, tcName);
			if (pLastAttribute == 0L) _pAttributes = new DtaScriptVariableList(pVariable);
			else pLastAttribute->pushBack(pVariable);
		}
		pVariable->copyAll(*(pVariableAttributes->getNode()), bMerge);
		pVariableAttributes = pVariableAttributes->getNext();
	}
	if (_pArrayCache == 0L) {
		_pArrayCache = new std::map<std::string, DtaScriptVariable*>();
		for (std::list<DtaScriptVariable*>::const_iterator i = _array.begin(); i != _array.end(); i++) (*_pArrayCache)[(*i)->_tcName] = (*i);
	}
    for (std::list<DtaScriptVariable*>::const_iterator i = variable._array.begin(); i != variable._array.end(); i++) {
		const char* tcName = (*i)->_tcName;
		std::map<std::string, DtaScriptVariable*>::const_iterator cursor = _pArrayCache->find(tcName);
		DtaScriptVariable* pVariable = 0L;
		if (cursor != _pArrayCache->end()) {
			pVariable = cursor->second;
		} else {
			pVariable = new DtaScriptVariable(this, tcName);
			_array.push_back(pVariable);
			(*_pArrayCache)[tcName] = pVariable;
		}
		pVariable->copyAll(*(*i), bMerge);
	}
}

DtaScriptVariable* DtaScriptVariable::pushItem(const std::string& sValue) {
	char tcNumber[32];
	sprintf(tcNumber, "%d", _array.size());
	std::string sNumber = tcNumber;
	DtaScriptVariable* pVariable = new DtaScriptVariable(this, sNumber);
	_array.push_back(pVariable);
	if (_pArrayCache != 0L) {
		(*_pArrayCache)[sNumber] = pVariable;
	}
	pVariable->setValue(sValue.c_str());
	return pVariable;
}

void DtaScriptVariable::slideNodeContent(DtaScriptVariable& visibility, const ExprScriptVariable& varExpr) {
	DtaScriptVariable* pTemporary = new DtaScriptVariable;
	pTemporary->copyAll(*this);
	clearContent();
	DtaScriptVariable* pDestVariable = getNonRecursiveVariable(visibility, varExpr);
	DtaScriptVariable* pParent = pDestVariable->getParent();
	DtaScriptVariableList* pParentAttributes = pParent->_pAttributes;
    while (pParentAttributes != 0L) {
	    if (pParentAttributes->getNode() == pDestVariable) {
			pParentAttributes->setNode(pTemporary);
			free((char*) pTemporary->_tcName);
			pTemporary->_tcName = strdup(pDestVariable->_tcName);
			pTemporary->_pParent = pParent;
		    break;
		}
		pParentAttributes = pParentAttributes->getNext();
	}
	if (pParentAttributes == 0L) {
		std::list<DtaScriptVariable*>::iterator i;
		for (i = pParent->_array.begin(); i != pParent->_array.end(); i++) {
			if ((*i) == pDestVariable) {
				(*i) = pTemporary;
				free((char*) pTemporary->_tcName);
				pTemporary->_tcName = strdup(pDestVariable->_tcName);
				pTemporary->_pParent = pParent;
				break;
			}
		}
		if (i == pParent->_array.end()) {
			delete pTemporary;
			throw UtlException("internal error: unable to slide a node content because destination node isn't attached to its parent properly");
		}
	}
	delete pDestVariable;
}

void DtaScriptVariable::trace(DtaScriptVariable& visibility) {
	CGRuntime::traceLine("Tracing variable '" + getCompleteName() + "':");
	if (!_sConstantValue.empty()) CGRuntime::traceLine("\t\"" + _sConstantValue + "\"");
	else if (_pVariableValue != 0L) CGRuntime::traceLine("\t-> '" + _pVariableValue->getCompleteName() + "' = \"" + _pVariableValue->getValue() + "\"");
	DtaScriptVariableList* pAttributes = _pAttributes;
    while (pAttributes != 0L) {
		DtaScriptVariable* pNode = pAttributes->getNode();
		CGRuntime::traceLine("\t" + std::string(pNode->_tcName) + " = \"" + pNode->getValue() + "\"");
		if (!pNode->_array.empty()) {
			CGRuntime::traceText("\t" + std::string(pNode->_tcName) + "[");
			for (std::list<DtaScriptVariable*>::iterator j = pNode->_array.begin(); j != pNode->_array.end(); j++) {
				if (j != pNode->_array.begin()) CGRuntime::traceText(", ");
				CGRuntime::traceText("\"" + std::string((*j)->_tcName) + "\"");
			}
			CGRuntime::traceLine("]");
		}
		pAttributes = pAttributes->getNext();
	}
	if (!_array.empty()) {
		CGRuntime::traceText("\t" + std::string(_tcName) + "[");
		for (std::list<DtaScriptVariable*>::iterator k = _array.begin(); k != _array.end(); k++) {
			if (k != _array.begin()) CGRuntime::traceText(", ");
			CGRuntime::traceText("\"" + std::string((*k)->_tcName) + "\"");
			if (!(*k)->getValue().empty()) CGRuntime::traceText(" -> \"" + (*k)->getValue() + "\"");
		}
		CGRuntime::traceLine("]");
	}
	CGRuntime::traceLine("End of variable's trace '" + getCompleteName() + "'.");
}

void DtaScriptVariable::traceStack() {
	if (strcmp(_tcName, "project") == 0) return;
	if (_pParent != 0) _pParent->traceStack();
	CGRuntime::traceLine(std::string(_tcName) + ":");
	DtaScriptVariableList* pAttributes = _pAttributes;
    while (pAttributes != 0L) {
		DtaScriptVariable* pNode = pAttributes->getNode();
		if (pNode->getReferencedVariable() != 0L) {
			std::string sReference = pNode->getReferencedVariable()->getCompleteName();
			CGRuntime::traceLine(std::string(pNode->_tcName) + " = " + sReference);
		} else {
			std::string sValue = pNode->getValue();
			CGRuntime::traceLine(std::string(pNode->_tcName) + " = \"" + sValue + "\"");
		}
		if (!pNode->_array.empty()) {
			CGRuntime::traceText("\t" + std::string(pNode->_tcName) + "[");
			for (std::list<DtaScriptVariable*>::iterator j = pNode->_array.begin(); j != pNode->_array.end(); j++) {
				if (j != pNode->_array.begin()) CGRuntime::traceText(", ");
				CGRuntime::traceText("\"" + std::string((*j)->_tcName) + "\"");
			}
			CGRuntime::traceLine("]");
		}
		pAttributes = pAttributes->getNext();
	}
	if (!_array.empty()) {
		CGRuntime::traceText("\t" + std::string(_tcName) + "[");
		for (std::list<DtaScriptVariable*>::iterator k = _array.begin(); k != _array.end(); k++) {
			if (k != _array.begin()) CGRuntime::traceText(", ");
			CGRuntime::traceText("\"" + std::string((*k)->_tcName) + "\"");
		}
		CGRuntime::traceLine("]");
	}
}



DtaScriptIndex::~DtaScriptIndex() {}
bool DtaScriptIndex::isIndex() const { return true; }
