/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include <math.h>

#include "UtlException.h"
#include "DtaScriptVariable.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"
#include "ExprScriptExpression.h"
#include "DtaScript.h"


namespace CodeWorker {
	ExprScriptExpression::~ExprScriptExpression() {}

	DtaScriptVariable* ExprScriptExpression::getReferencedVariable() const { return 0L; }

	double ExprScriptExpression::getDoubleValue(DtaScriptVariable& visibility) const {
		std::string sValue = getValue(visibility);
		return atof(sValue.c_str());
	}

	int ExprScriptExpression::getIntValue(DtaScriptVariable& visibility) const {
		std::string sValue = getValue(visibility);
		return atoi(sValue.c_str());
	}

	EXPRESSION_RETURN_TYPE ExprScriptExpression::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "<unhandled expression>";
		return NO_TYPE;
	}

	bool ExprScriptExpression::compileCppBoolean(CppCompilerEnvironment& theCompilerEnvironment, bool bNegative) const {
		if (bNegative) theCompilerEnvironment.getBody() << "!";
		theCompilerEnvironment.getBody() << "CGRuntime::toBoolean(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppInt(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "CGRuntime::toInteger(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppDouble(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "CGRuntime::toDouble(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppString(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "CGRuntime::toString(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppUnsignedLong(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "CGRuntime::toUnsignedLong(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppUnsignedShort(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "CGRuntime::toUnsignedShort(";
		compileCpp(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return false;
	}

	bool ExprScriptExpression::compileCppExpr(CppCompilerEnvironment& theCompilerEnvironment) const {
		char tcNumber[32];
		sprintf(tcNumber, "%d", theCompilerEnvironment.getBody().getOutputLocation());
		std::string sVariable = "_compilerExpression_";
		sVariable += tcNumber;
		std::string sDeclaration = "\tstatic CppParsingTree_expr " + sVariable + "(\"" + toString() + "\");" + ScpStream::ENDL;
		theCompilerEnvironment.getBody().insertText(sDeclaration, theCompilerEnvironment.getPointerToDeclarations());
		theCompilerEnvironment.getBody() << sVariable;
		return false;
	}

	std::string ExprScriptExpression::toString() const {
		throw UtlException("ExprScriptExpression::toString() <to redefine in subclasses>");
	}


	ExprScriptReference::~ExprScriptReference() {}
	std::string ExprScriptReference::getValue(DtaScriptVariable& visibility) const {
		return ((_pRef != 0L) ? _pRef->getValue(visibility) : _pVariable->getValue());
	}

	DtaScriptVariable* ExprScriptReference::getReferencedVariable() const { return _pVariable; }



	ExprScriptUnaryMinus::~ExprScriptUnaryMinus() { delete _pExpression; }

	std::string ExprScriptUnaryMinus::getValue(DtaScriptVariable& visibility) const {
		return CGRuntime::convertDoubleToString(getDoubleValue(visibility));
	}

	double ExprScriptUnaryMinus::getDoubleValue(DtaScriptVariable& visibility) const {
		return -(_pExpression->getDoubleValue(visibility));
	}

	std::string ExprScriptUnaryMinus::toString() const {
		return "-(" + _pExpression->toString() + ")";
	}

	EXPRESSION_RETURN_TYPE ExprScriptUnaryMinus::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		compileCppDouble(theCompilerEnvironment);
		return DOUBLE_TYPE;
	}

	bool ExprScriptUnaryMinus::compileCppDouble(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "-(";
		_pExpression->compileCppDouble(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return true;
	}


	ExprScriptBitwiseNot::~ExprScriptBitwiseNot() { delete _pExpression; }

	std::string ExprScriptBitwiseNot::getValue(DtaScriptVariable& visibility) const {
		return CGRuntime::convertIntToString(getIntValue(visibility));
	}

	int ExprScriptBitwiseNot::getIntValue(DtaScriptVariable& visibility) const {
		return ~(_pExpression->getIntValue(visibility));
	}

	std::string ExprScriptBitwiseNot::toString() const {
		return "~(" + _pExpression->toString() + ")";
	}

	EXPRESSION_RETURN_TYPE ExprScriptBitwiseNot::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		compileCppInt(theCompilerEnvironment);
		return INT_TYPE;
	}

	bool ExprScriptBitwiseNot::compileCppInt(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "~(";
		_pExpression->compileCppInt(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
		return true;
	}


	ExprScriptConstant::ExprScriptConstant(bool bValue) {
		if (bValue) _sValue = "true";
	}

	ExprScriptConstant::ExprScriptConstant(int iValue) {
		_sValue = CGRuntime::convertIntToString(iValue);
	}

	ExprScriptConstant::ExprScriptConstant(double dValue) {
		_sValue = CGRuntime::convertDoubleToString(dValue);
	}

	ExprScriptConstant::~ExprScriptConstant() {}
	std::string ExprScriptConstant::getValue(DtaScriptVariable&) const { return _sValue; }
	double ExprScriptConstant::getDoubleValue(DtaScriptVariable& visibility) const { return atof(_sValue.c_str()); }
	int ExprScriptConstant::getIntValue(DtaScriptVariable& visibility) const { return atoi(_sValue.c_str()); }

	std::string ExprScriptConstant::toString() const {
		std::string sText;
		char* u = (char*) _sValue.c_str();
		sText = "\"";
		while (*u != '\0') {
			switch(*u) {
				case '\\': sText += "\\\\";break;
				case '\"': sText += "\\\"";break;
				case '\'': sText += "\\\'";break;
				case '\a': sText += "\\a";break;
				case '\b': sText += "\\b";break;
				case '\f': sText += "\\f";break;
				case '\n': sText += "\\n";break;
				case '\r': sText += "\\r";break;
				case '\t': sText += "\\t";break;
				case '\v': sText += "\\v";break;
				default:
					sText += *u;
			}
			u++;
		}
		sText += "\"";
		return sText;
	}

	EXPRESSION_RETURN_TYPE ExprScriptConstant::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		compileCppString(theCompilerEnvironment);
		return STRING_TYPE;
	}

	bool ExprScriptConstant::compileCppBoolean(CppCompilerEnvironment& theCompilerEnvironment, bool bNegative) const {
		if (_sValue.empty() == bNegative) theCompilerEnvironment.getBody() << "true";
		else theCompilerEnvironment.getBody() << "false";
		return true;
	}

	bool ExprScriptConstant::compileCppInt(CppCompilerEnvironment& theCompilerEnvironment) const {
		if (_sValue.empty()) theCompilerEnvironment.getBody() << 0;
		else theCompilerEnvironment.getBody() << _sValue;
		return true;
	}

	bool ExprScriptConstant::compileCppDouble(CppCompilerEnvironment& theCompilerEnvironment) const {
		if (_sValue.empty()) theCompilerEnvironment.getBody() << "0.0";
		else theCompilerEnvironment.getBody() << _sValue;
		return true;
	}

	bool ExprScriptConstant::compileCppString(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody().writeString(_sValue);
		return true;
	}


	ExprScriptConstantTree::~ExprScriptConstantTree() {
		for (std::list<ExprScriptExpression*>::iterator i = _listOfItems.begin(); i != _listOfItems.end(); ++i) {
			delete *i;
		}
	}
	
	std::string ExprScriptConstantTree::getValue(DtaScriptVariable& /*visibility*/) const {
		throw UtlException("ExprScriptConstantTree::getValue() -> no sense for a constant tree!");
	}

	void ExprScriptConstantTree::setTree(DtaScriptVariable& visibility, DtaScriptVariable&  theVariable) const {
		for (std::list<ExprScriptExpression*>::const_iterator i = _listOfItems.begin(); i != _listOfItems.end(); ++i) {
			ExprScriptConstantTree* pConstantTree = dynamic_cast<ExprScriptConstantTree*>(*i);
			if (pConstantTree != 0L) {
				DtaScriptVariable* pItem = theVariable.pushItem("");
				pConstantTree->setTree(visibility, *pItem);
			} else {
				theVariable.pushItem((*i)->getValue(visibility));
			}
		}
	}

	EXPRESSION_RETURN_TYPE ExprScriptConstantTree::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		theCompilerEnvironment.getBody() << "<unhandled 'ExprScriptConstantTree'>";
		return NO_TYPE;
	}

	std::string ExprScriptConstantTree::toString() const {
		std::string sText = "{";
		for (std::list<ExprScriptExpression*>::const_iterator i = _listOfItems.begin(); i != _listOfItems.end(); ++i) {
			if (i != _listOfItems.begin()) sText += ", ";
			sText += (*i)->toString();
		}
		return sText + "}";
	}


	ExprScriptResolvedVariable::~ExprScriptResolvedVariable() {
	}

	std::string ExprScriptResolvedVariable::getValue(DtaScriptVariable&) const {
		return _pVariable->getValue();
	}

	std::string ExprScriptResolvedVariable::toString() const {
		return _pVariable->getCompleteName();
	}


	ExprScriptScriptFile::~ExprScriptScriptFile() {
		if (_bFileName) delete script._pFileName;
		else delete script._pBody;
	}

	std::string ExprScriptScriptFile::getValue(DtaScriptVariable& visibility) const {
		if (_bFileName) return script._pFileName->getValue(visibility);
		throw UtlException("Internal error: cannot ask for the value of a piece of script!");
	}

	std::string ExprScriptScriptFile::toString() const {
		if (_bFileName) return script._pFileName->toString();
		return "<piece of script>";
	}
}
