/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning(disable: 4786)
#endif

#include "DtaScriptVariable.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "ExprScriptVariable.h"

using namespace CodeWorker;

unsigned char ExprScriptVariable::ATTRIBUTE_ACCESS = 1;
unsigned char ExprScriptVariable::ARRAY_ACCESS = 2;
unsigned char ExprScriptVariable::ARRAY_KEY_ACCESS = 6;
unsigned char ExprScriptVariable::ARRAY_POSITION_ACCESS = 10;
unsigned char ExprScriptVariable::NEXT_ACCESS = 16;
unsigned char ExprScriptVariable::EVALUATION_ACCESS = 32;

ExprScriptVariable::~ExprScriptVariable() {
    if (_pNext != 0L) delete _pNext;
    if (_pExpression != 0L) delete _pExpression;
}

std::string ExprScriptVariable::getValue(DtaScriptVariable& visibility) const {
    DtaScriptVariable* pVariable = visibility.getExistingVariable(*this);
	if (pVariable == 0L) return "";
    return pVariable->getValue();
}

EXPRESSION_RETURN_TYPE ExprScriptVariable::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	if (isEvaluation()) {
		if (theCompilerEnvironment.getCurrentClause() != 0L) theCompilerEnvironment.getBody() << "_compilerClauseScope.getEvaluatedNode(";
		else theCompilerEnvironment.getBody() << "CGRuntime::getThisTree().getEvaluatedNode(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (_sName == "this") {
		theCompilerEnvironment.getBody() << "CGRuntime::getThisTree()";
	} else if (_sName == "project") {
		theCompilerEnvironment.getBody() << "CGRuntime::getRootTree()";
	} else if (theCompilerEnvironment.existVariable(_sName)) {
		theCompilerEnvironment.getBody() << _sName;
	} else if (theCompilerEnvironment.getCurrentClause() != 0L) {
		theCompilerEnvironment.getBody() << "_compilerClauseScope.getNode(\"" << _sName << "\")";
	} else {
		theCompilerEnvironment.getBody() << "CGRuntime::getThisTree().getNode(\"" << _sName << "\")";
	}
	compileCppFollowing(theCompilerEnvironment);
	return TREE_TYPE;
}

bool ExprScriptVariable::compileCppBoolean(CppCompilerEnvironment& theCompilerEnvironment, bool bNegative) const {
	if (!bNegative) theCompilerEnvironment.getBody() << "!";
	compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ".getValue().empty()";
	return true;
}

bool ExprScriptVariable::compileCppInt(CppCompilerEnvironment& theCompilerEnvironment) const {
	compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ".getIntValue()";
	return true;
}

bool ExprScriptVariable::compileCppDouble(CppCompilerEnvironment& theCompilerEnvironment) const {
	compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ".getDoubleValue()";
	return true;
}

bool ExprScriptVariable::compileCppString(CppCompilerEnvironment& theCompilerEnvironment) const {
	compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ".getValue()";
	return true;
}

void ExprScriptVariable::compileCppFollowing(CppCompilerEnvironment& theCompilerEnvironment) const {
	if (isEvaluation()) {
		theCompilerEnvironment.getBody() << ".getEvaluatedNode(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (isArrayKey()) {
		theCompilerEnvironment.getBody() << ".getArrayNodeFromKey(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (isArrayPosition()) {
		ExprScriptConstant* pConstant = dynamic_cast<ExprScriptConstant*>(_pExpression);
		if (pConstant != 0L) {
			std::string sConstant = pConstant->getConstant();
			if (sConstant == "front") theCompilerEnvironment.getBody() << ".getFirstArrayNode()";
			else if (sConstant == "back") theCompilerEnvironment.getBody() << ".getLastArrayNode()";
			else if (sConstant == "parent") theCompilerEnvironment.getBody() << ".getParentNode()";
		} else {
			theCompilerEnvironment.getBody() << ".getArrayNodeFromPosition(";
			_pExpression->compileCppInt(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ")";
		}
	}
	if (_pNext != 0L) {
		theCompilerEnvironment.getBody() << ".getNode(\"" << _pNext->_sName << "\")";
		_pNext->compileCppFollowing(theCompilerEnvironment);
	}
}

void ExprScriptVariable::compileCppForSet(CppCompilerEnvironment& theCompilerEnvironment) const {
	if (isEvaluation()) {
		if (theCompilerEnvironment.getCurrentClause() != 0L) theCompilerEnvironment.getBody() << "_compilerClauseScope.insertClassicalEvaluatedNode(";
		else theCompilerEnvironment.getBody() << "CGRuntime::getThisTree().insertEvaluatedNode(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (_sName == "this") {
		theCompilerEnvironment.getBody() << "CGRuntime::getThisTree()";
	} else if (_sName == "project") {
		theCompilerEnvironment.getBody() << "CGRuntime::getRootTree()";
	} else if (theCompilerEnvironment.existVariable(_sName)) {
		theCompilerEnvironment.getBody() << _sName;
	} else if (theCompilerEnvironment.getCurrentClause() != 0L) {
		theCompilerEnvironment.getBody() << "_compilerClauseScope.insertClassicalNode(\"" << _sName << "\")";
	} else {
		theCompilerEnvironment.getBody() << "CGRuntime::getThisTree().insertNode(\"" << _sName << "\")";
	}
	compileCppFollowingForSet(theCompilerEnvironment);
}

void ExprScriptVariable::compileCppForBNFSet(CppCompilerEnvironment& theCompilerEnvironment) const {
	if (isEvaluation()) {
		theCompilerEnvironment.getBody() << "_compilerClauseScope.insertBNFEvaluatedNode(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (_sName == "this") {
		theCompilerEnvironment.getBody() << "CGRuntime::getThisTree()";
	} else if (_sName == "project") {
		theCompilerEnvironment.getBody() << "CGRuntime::getRootTree()";
	} else {
		theCompilerEnvironment.getBody() << "_compilerClauseScope.insertBNFNode(\"" << _sName << "\")";
	}
	compileCppFollowingForSet(theCompilerEnvironment);
}

void ExprScriptVariable::compileCppFollowingForSet(CppCompilerEnvironment& theCompilerEnvironment) const {
	if (isEvaluation()) {
		theCompilerEnvironment.getBody() << ".insertEvaluatedNode(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (isArrayKey()) {
		theCompilerEnvironment.getBody() << ".insertArrayNodeFromKey(";
		_pExpression->compileCppString(theCompilerEnvironment);
		theCompilerEnvironment.getBody() << ")";
	} else if (isArrayPosition()) {
		ExprScriptConstant* pConstant = dynamic_cast<ExprScriptConstant*>(_pExpression);
		if (pConstant != 0L) {
			std::string sConstant = pConstant->getConstant();
			if (sConstant == "front") theCompilerEnvironment.getBody() << ".getFirstArrayNode()";
			else if (sConstant == "back") theCompilerEnvironment.getBody() << ".getLastArrayNode()";
			else if (sConstant == "parent") theCompilerEnvironment.getBody() << ".getParentNode()";
		} else {
			theCompilerEnvironment.getBody() << ".insertArrayNodeFromPosition(";
			_pExpression->compileCppInt(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ")";
		}
	}
	if (_pNext != 0L) {
		theCompilerEnvironment.getBody() << ".insertNode(\"" << _pNext->_sName << "\")";
		_pNext->compileCppFollowingForSet(theCompilerEnvironment);
	}
}

bool ExprScriptVariable::compileCppExpr(CppCompilerEnvironment& theCompilerEnvironment) const {
	char tcNumber[32];
	sprintf(tcNumber, "%d", theCompilerEnvironment.getBody().getOutputLocation());
	std::string sVariable = "_compilerVariable_";
	sVariable += tcNumber;
	std::string sDeclaration = "\tstatic CppParsingTree_varexpr " + sVariable + "(\"" + toString() + "\");" + ScpStream::ENDL;
	theCompilerEnvironment.getBody().insertText(sDeclaration, theCompilerEnvironment.getPointerToDeclarations());
	theCompilerEnvironment.getBody() << sVariable;
	return false;
}

std::string ExprScriptVariable::toString() const {
	std::string sText;
	if (isEvaluation()) {
		sText = "#evaluateVariable(" + _pExpression->toString() + ")";
	} else {
		sText = _sName;
		if (isArrayKey()) {
			sText += "[" + _pExpression->toString() + "]";
		} else if (isArrayPosition()) {
			std::string sPosition = _pExpression->toString();
			if ((sPosition == "front") || (sPosition == "back") || (sPosition == "parent")) sText += "#" + sPosition;
			else sText += "#[" + sPosition + "]";
		}
	}
	if (_pNext != 0L) {
		sText += "." + _pNext->toString();
	}
	return sText;
}

