/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "GrfForeach.h"

using namespace CodeWorker;

GrfForeach::~GrfForeach() {
	delete _pIndexExpr;
	delete _pListExpr;
}

void GrfForeach::setSortedNoCase(bool bSortedNoCase) {
	if (bSortedNoCase) _bSorted = true;
	_bSortedNoCase = bSortedNoCase;
}

void GrfForeach::setReverse(bool bReverse) {
	_bReverse = bReverse;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeInternal(DtaScriptVariable& visibility) {
	SEQUENCE_INTERRUPTION_LIST result = NO_INTERRUPTION;
	DtaScriptVariable* pVariable = visibility.getExistingVariable(*_pListExpr);
	if (pVariable != 0L) {
		if (!pVariable->getArray().empty()) {
			DtaScriptVariable stackForeach(&visibility, "##stack## foreach");
			DtaScriptIndex* pIndex = stackForeach.createIndex(_pIndexExpr->getName().c_str());
			pIndex->setForeach(this);
			if (_iCascading != NO_CASCADING) _sCascadedName = pVariable->getName();
			if (_bSorted) {
				if (_bSortedNoCase) {
					if (_bReverse) result = executeReverseSortedNoCaseForeach(*pIndex, *pVariable, stackForeach);
					else result = executeSortedNoCaseForeach(*pIndex, *pVariable, stackForeach);
				} else {
					if (_bReverse) result = executeReverseSortedForeach(*pIndex, *pVariable, stackForeach);
					else result = executeSortedForeach(*pIndex, *pVariable, stackForeach);
				}
			} else {
				if (_bReverse) result = executeReverseForeach(*pIndex, *pVariable, stackForeach);
				else result = executeForeach(*pIndex, *pVariable, stackForeach);
			}
			if (result == BREAK_INTERRUPTION) result = NO_INTERRUPTION;
		}
	}
	return result;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	const std::list<DtaScriptVariable*>& list = theVariable.getArray();
	int iOldArraySize = _iArraySize;
	_iArraySize = list.size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::list<DtaScriptVariable*>::const_iterator i = list.begin(); i != list.end(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue((*i));
		_sKey = (*i)->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	_sKey = sOldKey;
	return NO_INTERRUPTION;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeReverseForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	const std::list<DtaScriptVariable*>& list = theVariable.getArray();
	int iOldArraySize = _iArraySize;
	_iArraySize = list.size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::list<DtaScriptVariable*>::const_reverse_iterator i = list.rbegin(); i != list.rend(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue((*i));
		_sKey = (*i)->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	_sKey = sOldKey;
	return NO_INTERRUPTION;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeSortedForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	const std::map<std::string, DtaScriptVariable*>& mapOfElements = theVariable.getSortedArray();
	int iOldArraySize = _iArraySize;
	_iArraySize = mapOfElements.size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::map<std::string, DtaScriptVariable*>::const_iterator i = mapOfElements.begin(); i != mapOfElements.end(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = i->second->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue(i->second);
		_sKey = i->second->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = i->second->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_sKey = sOldKey;
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	return NO_INTERRUPTION;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeSortedNoCaseForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	std::auto_ptr<std::vector<DtaScriptVariable*> > listOfElements(theVariable.getSortedNoCaseArray());
	int iOldArraySize = _iArraySize;
	_iArraySize = listOfElements->size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::vector<DtaScriptVariable*>::const_iterator i = listOfElements->begin(); i != listOfElements->end(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedNoCaseForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue(*i);
		_sKey = (*i)->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedNoCaseForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_sKey = sOldKey;
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	return NO_INTERRUPTION;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeReverseSortedForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	const std::map<std::string, DtaScriptVariable*>& mapOfElements = theVariable.getSortedArray();
	int iOldArraySize = _iArraySize;
	_iArraySize = mapOfElements.size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::map<std::string, DtaScriptVariable*>::const_reverse_iterator i = mapOfElements.rbegin(); i != mapOfElements.rend(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = i->second->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue(i->second);
		_sKey = i->second->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = i->second->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_sKey = sOldKey;
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	return NO_INTERRUPTION;
}

SEQUENCE_INTERRUPTION_LIST GrfForeach::executeReverseSortedNoCaseForeach(DtaScriptIndex& theIndex, DtaScriptVariable& theVariable, DtaScriptVariable& stackForeach) {
	SEQUENCE_INTERRUPTION_LIST result;
	std::auto_ptr<std::vector<DtaScriptVariable*> > listOfElements(theVariable.getSortedNoCaseArray());
	int iOldArraySize = _iArraySize;
	_iArraySize = listOfElements->size();
	int iOldPosition = _iCurrentPosition;
	_iCurrentPosition = 0;
	std::string sOldKey = _sKey;
	for (std::vector<DtaScriptVariable*>::reverse_iterator i = listOfElements->rbegin(); i != listOfElements->rend(); i++) {
		if (_iCascading == CASCADING_FIRST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedNoCaseForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		theIndex.setValue(*i);
		_sKey = (*i)->getName();
		result = GrfBlock::executeInternal(stackForeach);
		if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
		if (_iCascading == CASCADING_LAST) {
			DtaScriptVariable* pNext = (*i)->getNode(_sCascadedName.c_str());
			if (pNext != 0L) {
				result = executeSortedNoCaseForeach(theIndex, *pNext, stackForeach);
				if ((result != CONTINUE_INTERRUPTION) && (result != NO_INTERRUPTION)) return result;
			}
		}
		_iCurrentPosition++;
	}
	_sKey = sOldKey;
	_iCurrentPosition = iOldPosition;
	_iArraySize = iOldArraySize;
	return NO_INTERRUPTION;
}

void GrfForeach::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	std::string sIndex = _pIndexExpr->getName();
	if (_bSorted) {
		if (_bSortedNoCase) {
			if (theCompilerEnvironment.addVariable("_compilerMapSortedNoCaseForeach_" + sIndex)) {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::vector<DtaScriptVariable*>::const_iterator _compilerIndexSortedNoCaseForeach_" << sIndex << ";";
				theCompilerEnvironment.getBody().endl();
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::auto_ptr<std::vector<DtaScriptVariable*> > _compilerMapSortedNoCaseForeach_" << sIndex << "(";
				_pListExpr->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".getSortedNoCaseArray());";
				theCompilerEnvironment.getBody().endl();
			} else {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerMapSortedNoCaseForeach_" << sIndex << " = ";
				_pListExpr->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".getSortedNoCaseArray();";
				theCompilerEnvironment.getBody().endl();
			}
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "for (_compilerIndexSortedNoCaseForeach_" << sIndex << " = _compilerMapSortedNoCaseForeach_" << sIndex << "->begin(); _compilerIndexSortedNoCaseForeach_" << sIndex << " != _compilerMapSortedNoCaseForeach_" << sIndex << "->end(); ++_compilerIndexSortedNoCaseForeach_" << sIndex << ") {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::string _compilerKeyForeach_" << sIndex << "(_compilerIndexSortedNoCaseForeach_" << sIndex << "->getName());";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CppParsingTree_var " << sIndex << "(*_compilerIndexSortedNoCaseForeach_" << sIndex << ");";
			theCompilerEnvironment.getBody().endl();
		} else {
			if (theCompilerEnvironment.addVariable(sIndex)) {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::map<std::string, DtaScriptVariable*>::const_iterator _compilerIndexForeach_" << sIndex << ";";
				theCompilerEnvironment.getBody().endl();
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "const std::map<std::string, DtaScriptVariable*>* _compilerMapForeach_" << sIndex << " = &(";
				_pListExpr->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".getSortedArray());";
				theCompilerEnvironment.getBody().endl();
			} else {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerMapForeach_" << sIndex << " = &(";
				_pListExpr->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ".getSortedArray());";
				theCompilerEnvironment.getBody().endl();
			}
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "for (_compilerIndexForeach_" << sIndex << " = _compilerMapForeach_" << sIndex << "->begin(); _compilerIndexForeach_" << sIndex << " != _compilerMapForeach_" << sIndex << "->end(); ++_compilerIndexForeach_" << sIndex << ") {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::string _compilerKeyForeach_" << sIndex << "(_compilerIndexForeach_" << sIndex << "->first());";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CppParsingTree_var " << sIndex << "(_compilerIndexForeach_" << sIndex << "->second);";
			theCompilerEnvironment.getBody().endl();
		}
	} else {
		if (theCompilerEnvironment.addVariable(sIndex)) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::list<DtaScriptVariable*>::const_iterator _compilerIndexForeach_" << sIndex << ";";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "const std::list<DtaScriptVariable*>* _compilerMapForeach_" << sIndex << " = &(";
			_pListExpr->compileCpp(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ".getArray());";
			theCompilerEnvironment.getBody().endl();
		} else {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerMapForeach_" << sIndex << " = &(";
			_pListExpr->compileCpp(theCompilerEnvironment);
			theCompilerEnvironment.getBody() << ".getArray());";
			theCompilerEnvironment.getBody().endl();
		}
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "for (_compilerIndexForeach_" << sIndex << " = _compilerMapForeach_" << sIndex << "->begin(); _compilerIndexForeach_" << sIndex << " != _compilerMapForeach_" << sIndex << "->end(); ++_compilerIndexForeach_" << sIndex << ") {";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.incrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::string _compilerKeyForeach_" << sIndex << "(CppParsingTree_var(*_compilerIndexForeach_" << sIndex << ").getName());";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CppParsingTree_var " << sIndex << "(*_compilerIndexForeach_" << sIndex << ");";
		theCompilerEnvironment.getBody().endl();
	}
	theCompilerEnvironment.bracketsToNextBlock(false);
	GrfBlock::compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
}

