/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "GrfBlock.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "GrfLocalVariable.h"

namespace CodeWorker {
	GrfLocalVariable::~GrfLocalVariable() {
		if (_pVariable != 0L) delete _pVariable;
		if (_pValue != 0L) delete _pValue;
	}

	SEQUENCE_INTERRUPTION_LIST GrfLocalVariable::executeInternal(DtaScriptVariable& visibility) {
		DtaScriptVariable* pLocalVariable = visibility.getNonRecursiveVariable(visibility, *_pVariable);
		pLocalVariable->clearContent();
		if (_pValue != 0L) {
			ExprScriptConstantTree* pConstantTree = dynamic_cast<ExprScriptConstantTree*>(_pValue);
			if (pConstantTree != 0L) {
				pConstantTree->setTree(visibility, *pLocalVariable);
			} else {
				std::string sValue = _pValue->getValue(visibility);
				pLocalVariable->setValue(sValue.c_str());
			}
		}
		return NO_INTERRUPTION;
	}

	void GrfLocalVariable::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
		if (!theCompilerEnvironment.addVariable(_pVariable->getName())) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << _pVariable->getName() << ".clearNode();";
			theCompilerEnvironment.getBody().endl();
			if (_pValue != 0L) {
				theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << _pVariable->getName() << ".setValue(";
				_pValue->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ");";
			theCompilerEnvironment.getBody().endl();
			}
		} else {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CppParsingTree_value " << _pVariable->getName();
			if (_pValue != 0L) {
				theCompilerEnvironment.getBody() << "(";
				_pValue->compileCpp(theCompilerEnvironment);
				theCompilerEnvironment.getBody() << ");";
			} else {
				theCompilerEnvironment.getBody() << ";";
			}
			theCompilerEnvironment.getBody().endl();
		}
	}
}
