/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"

#include "ExprScriptExpression.h"
#include "ExprScriptVariable.h"
#include "DtaBNFScript.h"
#include "DtaScriptVariable.h"
#include "BNFClause.h"
#include "DtaProject.h"
#include "GrfParseAsBNF.h"

using namespace CodeWorker;

GrfParseAsBNF::~GrfParseAsBNF() {
	delete _pBNFFileName;
	delete _pClass;
	delete _pFileName;
	if (_pCachedScript != 0L) delete _pCachedScript;
}

void GrfParseAsBNF::setBNFFileName(ExprScriptScriptFile* pBNFFileName) {
	if (pBNFFileName->isFileName()) _pBNFFileName = pBNFFileName->getFileName();
	_pCachedScript = dynamic_cast<DtaBNFScript*>(pBNFFileName->getBody());
	pBNFFileName->release();
}

SEQUENCE_INTERRUPTION_LIST GrfParseAsBNF::executeInternal(DtaScriptVariable& visibility) {
	DtaScriptVariable* pClass = visibility.getExistingVariable(*_pClass);
	if (pClass == 0L) throw UtlException("runtime error: variable '" + _pClass->toString() + "' doesn't exist while calling procedure 'parseAsBNF()'");
	std::string sOutputFile = _pFileName->getValue(visibility);

	if (_pBNFFileName != 0L) {
		std::string sBNFFileName = _pBNFFileName->getValue(visibility);
		if ((_pCachedScript == 0L) || (_sCachedBNFFile != sBNFFileName) || ScpStream::existVirtualFile(sBNFFileName)) {
			if (_pCachedScript != 0L) delete _pCachedScript;
			_pCachedScript = new DtaBNFScript(getParent());
			_sCachedBNFFile = sBNFFileName;
			_pCachedScript->parseFile(sBNFFileName.c_str());
		}
	}

	SEQUENCE_INTERRUPTION_LIST result = _pCachedScript->generate(sOutputFile.c_str(), *pClass);
	switch(result) {
		case CONTINUE_INTERRUPTION:
		case BREAK_INTERRUPTION:
		case RETURN_INTERRUPTION:
			result = NO_INTERRUPTION;
			break;
	}
	return result;
}

void GrfParseAsBNF::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	ExprScriptConstant* pConstantFileName = dynamic_cast<ExprScriptConstant*>(_pBNFFileName);
	if (pConstantFileName == 0L) throw UtlException("constant script filename expected while compiling 'parseAFree' procedure into script \"" + std::string(_sParsingFilePtr) + "\"");
	std::string sRadical = DtaScript::convertFilenameAsIdentifier(CppCompilerEnvironment::getRadical(pConstantFileName->getConstant()));
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CGRuntime::parseAsBNF(*execute" << sRadical << ", ";
	_pClass->compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ", ";
	_pFileName->compileCppString(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ");";
	theCompilerEnvironment.getBody().endl();
	if (_pCachedScript == 0L) {
		_pCachedScript = new DtaBNFScript(getParent());
		_sCachedBNFFile = pConstantFileName->getConstant();
		_pCachedScript->parseFile(_sCachedBNFFile.c_str());
	}
	_pCachedScript->compileCpp(theCompilerEnvironment, pConstantFileName->getConstant());
}
