/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"

#include "DtaScriptVariable.h"
#include "ExprScriptVariable.h"
#include "CGExternalHandling.h"
#include "CppCompilerEnvironment.h"
#include "GrfQuiet.h"

using namespace CodeWorker;

GrfQuiet::~GrfQuiet() {
	delete _pVariable;
}

SEQUENCE_INTERRUPTION_LIST GrfQuiet::executeInternal(DtaScriptVariable& visibility) {
	SEQUENCE_INTERRUPTION_LIST result;
	CGQuietOutput quiet;
	result = GrfBlock::executeInternal(visibility);
	DtaScriptVariable* pVariable = visibility.getExistingVariable(*_pVariable);
	if (pVariable == 0L) throw UtlException("runtime error: variable '" + _pVariable->toString() + "' should have been declared before using it in the 'quiet' statement modifier");
	pVariable->setValue(quiet.getOutput().c_str());
	return result;
}

void GrfQuiet::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "{";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.incrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CGQuietOutput quiet;";
	theCompilerEnvironment.getBody().endl();
	GrfBlock::compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation();
	_pVariable->compileCppForSet(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << ".setValue(quiet.getOutput());";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
}
