/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "DtaScriptVariable.h"
#include "ExprScriptExpression.h"
#include "GrfSwitch.h"

using namespace CodeWorker;

GrfSwitch::~GrfSwitch() {
	delete _pExpression;
}

SEQUENCE_INTERRUPTION_LIST GrfSwitch::executeInternal(DtaScriptVariable& visibility) {
	SEQUENCE_INTERRUPTION_LIST result = NO_INTERRUPTION;
	DtaScriptVariable stackSwitch(&visibility, "##stack## switch");
	std::string sExpression = _pExpression->getValue(visibility);
	if (equalString(sExpression.c_str())) result = GrfBlock::executeAt(stackSwitch, _directionEqualList[sExpression]);
	else {
		bool bFound = false;
		int iRank;
		for (std::map<std::string, int>::const_iterator i = _directionStartList.begin(); i != _directionStartList.end(); i++) {
			if (strncmp(i->first.c_str(), sExpression.c_str(), i->first.size()) == 0) {
				bFound = true;
				iRank = i->second;
			} else if (bFound) {
				break;
			}
		}
		if (bFound) {
			result = GrfBlock::executeAt(stackSwitch, iRank);
		} else {
			if (_iDefault >= 0) {
				result = GrfBlock::executeAt(stackSwitch, _iDefault);
			} else {
				throw UtlException("switch internal error: unknown '" + sExpression + "' case encountered, it lacks either a 'case' or a 'default' statement");
			}
		}
	}
	switch(result) {
		case BREAK_INTERRUPTION:
		case CONTINUE_INTERRUPTION:
			result = NO_INTERRUPTION;
		case NO_INTERRUPTION: break;
	}
	return result;
}

void GrfSwitch::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	std::map<int, std::string> mapOfStatements;
	int iSwitchNumber = theCompilerEnvironment.newSwitch();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::string _compilerSwitch_key" << iSwitchNumber << " = ";
	_pExpression->compileCppString(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << ";";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "int _compilerSwitch_statement" << iSwitchNumber << ";";
	theCompilerEnvironment.getBody().endl();
	if (!_directionEqualList.empty()) {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "static std::map<std::string, int> _compilerSwitch_table" << iSwitchNumber << ";";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerSwitch_table" << iSwitchNumber << ".empty()) {";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.incrementIndentation();
		for (std::map<std::string, int>::const_iterator i = _directionEqualList.begin(); i != _directionEqualList.end(); i++) {
			mapOfStatements[i->second] = i->first;
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerSwitch_table" << iSwitchNumber << "[";
			theCompilerEnvironment.getBody().writeString(i->first);
			theCompilerEnvironment.getBody() << "] = " << i->second << ";";
			theCompilerEnvironment.getBody().endl();
		}
		theCompilerEnvironment.decrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "std::map<std::string, int>::const_iterator _compilerSwitch_cursor" << iSwitchNumber << " = _compilerSwitch_table" << iSwitchNumber << ".find(_compilerSwitch_key" << iSwitchNumber << ");";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "if (_compilerSwitch_cursor" << iSwitchNumber << " != _compilerSwitch_table" << iSwitchNumber << ".end()) {";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.incrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerSwitch_statement" << iSwitchNumber << " = _compilerSwitch_cursor" << iSwitchNumber << "->second;";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.decrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	}
	if (!_directionStartList.empty()) {
		if (_directionEqualList.empty()) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation();
		} else {
			theCompilerEnvironment.getBody() << " else ";
		}
		for (std::map<std::string, int>::const_iterator i = _directionStartList.begin(); i != _directionStartList.end(); i++) {
			if (i != _directionStartList.begin()) {
				theCompilerEnvironment.getBody() << " else ";
			}
			mapOfStatements[i->second] = i->first;
			theCompilerEnvironment.getBody() << "if (strncmp(_compilerSwitch_key" << iSwitchNumber << ".c_str(), ";
			theCompilerEnvironment.getBody().writeString(i->first);
			theCompilerEnvironment.getBody() << ", " << (int) i->first.size() << ") == 0) {";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerSwitch_statement" << iSwitchNumber << " = " << i->second << ";";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.decrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
		}
	}
	if (_directionStartList.empty() && _directionEqualList.empty()) {
		throw UtlException("empty 'switch' statement encountered into file \"" + std::string(_sParsingFilePtr) + "\"");
	}
	theCompilerEnvironment.getBody() << " else {";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.incrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "_compilerSwitch_statement" << iSwitchNumber << " = " << _iDefault << ";";
	theCompilerEnvironment.getBody().endl();
	if (_iDefault >= 0) mapOfStatements[_iDefault] = "default";
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "switch(_compilerSwitch_statement" << iSwitchNumber << ") {";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.incrementIndentation();
	int iLastPosition = -1;
	for (std::map<int, std::string>::const_iterator i = mapOfStatements.begin(); i != mapOfStatements.end(); i++) {
		if (iLastPosition >= 0) {
			theCompilerEnvironment.incrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "{";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.incrementIndentation();
			compileCppBetween(theCompilerEnvironment, iLastPosition, i->first);
			theCompilerEnvironment.decrementIndentation();
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
			theCompilerEnvironment.getBody().endl();
			theCompilerEnvironment.decrementIndentation();
		}
		if (i->first == _iDefault) {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "default:";
		} else {
			theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "case " << i->first << ":";
		}
		theCompilerEnvironment.getBody().endl();
		iLastPosition = i->first;
	}
	theCompilerEnvironment.incrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "{";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.incrementIndentation();
	compileCppAt(theCompilerEnvironment, iLastPosition);
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.decrementIndentation();
	if (_iDefault < 0) {
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "default:";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.incrementIndentation();
		theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "throw UtlException(\"switch internal error: unknown '\" + _compilerSwitch_key" << iSwitchNumber << " + \"' case encountered into script \\\"" << _sParsingFilePtr << "\\\", it lacks either a 'case' or a 'default' statement\");";
		theCompilerEnvironment.getBody().endl();
		theCompilerEnvironment.decrementIndentation();
	}
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
}
