/* "CodeWorker":	a scripting language for parsing and generating text.

Copyright (C) 1996-1997, 1999-2002 Cdric Lemaire

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author: codeworker@free.fr
*/

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "UtlException.h"
#include "ExprScriptVariable.h"
#include "DtaScriptVariable.h"
#include "GrfBlock.h"
#include "ScpStream.h"
#include "CppCompilerEnvironment.h"
#include "CGRuntime.h"
#include "GrfTryCatch.h"

using namespace CodeWorker;

GrfTryCatch::~GrfTryCatch() {
	delete _pVariable;
	delete _pTryBlock;
	delete _pCatchBlock;
}

void GrfTryCatch::applyRecursively(APPLY_ON_COMMAND_FUNCTION apply) {
	GrfCommand::applyRecursively(apply);
	if (_pTryBlock != 0L) _pTryBlock->applyRecursively(apply);
	if (_pCatchBlock != 0L) _pCatchBlock->applyRecursively(apply);
}


SEQUENCE_INTERRUPTION_LIST GrfTryCatch::executeInternal(DtaScriptVariable& visibility) {
	SEQUENCE_INTERRUPTION_LIST result = NO_INTERRUPTION;
	try {
		result = _pTryBlock->execute(visibility);
	} catch(UtlException& exception) {
		DtaScriptVariable stackCatch(&visibility, "##stack## catch");
		DtaScriptVariable* pVariable = stackCatch.getNonRecursiveVariable(stackCatch, *_pVariable);
		std::string sError = exception.getMessage() + CGRuntime::endl() + exception.getTraceStack();
		pVariable->setValue(sError.c_str());
		result = _pCatchBlock->execute(stackCatch);
	}
	return result;
}

void GrfTryCatch::compileCpp(CppCompilerEnvironment& theCompilerEnvironment) const {
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "try ";
	theCompilerEnvironment.carriageReturnAfterBlock(false);
	_pTryBlock->compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.getBody() << " catch(UtlException& _compilerException_" << _pVariable->getName() << ") {";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.incrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "CppParsingTree_value " << _pVariable->getName() << "(_compilerException_" << _pVariable->getName() << ".getMessage());";
	theCompilerEnvironment.getBody().endl();
	theCompilerEnvironment.bracketsToNextBlock(false);
	_pCatchBlock->compileCpp(theCompilerEnvironment);
	theCompilerEnvironment.decrementIndentation();
	theCompilerEnvironment.getBody() << theCompilerEnvironment.getIndentation() << "}";
	theCompilerEnvironment.getBody().endl();
}
